using System;
using System.Collections.Generic;
using System.Text;
using NQueue.BusinessServices.Scheduling.ScheduleProcessors;
using NQueue.Entities;
using NQueue.DataServices;
using System.Runtime.Remoting;

namespace NQueue.BusinessServices.Scheduling
{
    public class ScheduleManager
    {
        /// <summary>
        /// Gets a list of all schedules currently in the database
        /// </summary>
        /// <returns></returns>
        public static List<ScheduleType> GetAllScheduleTypes()
        {
            return DBScheduling.GetAllScheduleTypes();
        }

        public static ScheduleProcessor GetScheduleProcessor(ScheduleType scheduleType)
        {
            ObjectHandle handle = Activator.CreateInstance(scheduleType.ScheduleTypeStrongName, scheduleType.ScheduleTypeClassName);
            return (ScheduleProcessor)handle.Unwrap();
        }


        public static ScheduleProcessor GetScheduleProcessor(int scheduleTypeID)
        {
            ScheduleType schedule = DBScheduling.GetSchedule(scheduleTypeID);
            return GetScheduleProcessor(schedule);
        }

        /// <summary>
        /// Inserts a schedule for a job instance into the database
        /// </summary>
        /// <param name="jobInstanceID">Job instance being scheduled - a job instance can have more that one schedule</param>
        /// <param name="ScheduleTypeID">Type ID of the schedule type - daily, monthly, etc</param>
        /// <param name="scheduleParameters">Instance of the relevant parameters object for the schedule type - to describe eg "the 3rd day of the week"</param>
        public static void InsertInstanceSchedule(int jobInstanceID, int scheduleTypeID, object scheduleParameters)
        {
            string scheduleParametersString = SerialisationHelper.SerialiseObject(scheduleParameters);
            DBJobInstance.InsertInstanceSchedule(jobInstanceID, scheduleTypeID, scheduleParametersString);
        }

        /// <summary>
        /// Deletes a job instance schedule item
        /// </summary>
        /// <param name="jobInstanceScheduleID"></param>
        public static void DeleteInstanceSchedule(JobInstanceSchedule jobInstanceSchedule)
        {
            DBJobInstance.DeleteInstanceSchedule(jobInstanceSchedule.JobInstanceScheduleID);
        }


    }
}
