using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using NQueue.Entities;

namespace NQueue.BusinessServices.Scheduling.ScheduleProcessors
{
    public class MinutelyScheduleProcessor : ScheduleProcessor
    {
        public MinutelyScheduleProcessor() : base() { ; }
        public MinutelyScheduleProcessor(int scheduleTypeID, int hostInstanceID)
            : base(scheduleTypeID, hostInstanceID)
        {
            ;
        }

        public override Type ParametersType
        {
            get { return typeof(MinutelyScheduleParameters); }
        }

        public override NQueue.Entities.ScheduleProcessingDueResults IsScheduleTypeDue(DateTime? lastRunDateTime, DateTime? anticipatedNextRunDateTime)
        {
            //The schedule type should be processed once a minute

            //if never run before, its due
            if (lastRunDateTime == null) return ScheduleProcessingDueResults.Due;

            //Time elapsed since the job last run
            double minutesSinceLastRun = DateTime.Now.Subtract((DateTime)lastRunDateTime).TotalMinutes;

            if (minutesSinceLastRun < 1) return NQueue.Entities.ScheduleProcessingDueResults.NotDue;
            else if (minutesSinceLastRun > 3) return NQueue.Entities.ScheduleProcessingDueResults.Overdue;
            else return NQueue.Entities.ScheduleProcessingDueResults.Due;
        }

        public override DateTime GetIndicativeNextRunDateTime()
        {
            //should run in 1 minutes time
            return DateTime.Now.AddMinutes(1);
        }

        protected override List<NQueue.Entities.WorkItemToEnqueue> ExecuteConcreteInstanceScheduleProcessing(NQueue.Entities.JobInstanceSchedule schedule, object scheduleParametersObject)
        {
            MinutelyScheduleParameters instanceParameters = (MinutelyScheduleParameters)scheduleParametersObject;
            List<WorkItemToEnqueue> itemsToEnqueue = new List<WorkItemToEnqueue>();
            //if the start date has passed and the previous process date is N minutes or more ago, enqueue one item to process immediately
            if (instanceParameters.StartDateAndTime.Date <= DateTime.Now)
            {
                if (schedule.InstanceScheduleLastRun != null && DateTime.Now.Subtract((DateTime)schedule.InstanceScheduleLastRun).TotalMinutes >= instanceParameters.EveryNMinutes)
                {
                    itemsToEnqueue.Add(new WorkItemToEnqueue(schedule.JobInstanceID, DateTime.Now));
                    schedule.JobInstanceScheduleWasJustProcessed = true;
                }
            }
            return itemsToEnqueue;
        }

        public class MinutelyScheduleParameters
        {
            private int _everyNMinutes;
            [Description("This schedule should run every n'th minute")]
            public int EveryNMinutes
            {
                get { return _everyNMinutes; }
                set { _everyNMinutes = value; }
            }

            private DateTime _startDateAndTime;
            [Description("Date and time before which processing should not start.")]
            public DateTime StartDateAndTime
            {
                get { return _startDateAndTime; }
                set { _startDateAndTime = value; }
            }

        }
    }
}
