using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using NQueue.DataServices;
using NQueue.Client;
using System.IO;

namespace NQueue.BusinessServices.System
{
    /// <summary>
    /// Manages system configuration
    /// </summary>
    public class SystemManager
    {
        /// <summary>
        /// Gets the entire system configuration
        /// </summary>
        /// <returns></returns>
        public static SystemConfiguration GetSystemConfiguration()
        {
            SystemConfiguration config = new SystemConfiguration();
            config.Hosts = DBHost.GetAllHosts();
            config.HostInstances = DBHostInstance.GetAllHostInstances();
            config.JobDefinitions = DBJobDefinition.GetAllJobDefinitions();
            config.JobInstances = DBJobInstance.GetAllJobInstances();
            
            return config;
        }

        /// <summary>
        /// Returns true if a server is authorised to run as a given host instance
        /// ID.  This stops rogue servers dequeuing work.
        /// </summary>
        /// <param name="hostInstanceID"></param>
        /// <returns></returns>
        public static bool CanServerRunHostInstance(int hostInstanceID)
        {
            HostInstance hostInstance = DBHostInstance.GetHostInstance(hostInstanceID);
            bool serverNamesMatch = hostInstance.ServerName.ToLower().Trim() == Environment.MachineName.ToLower().Trim();
            if (!serverNamesMatch)
            {
                string errorMessage = string.Format("NQueue service is running on a servername that does not match the recorded server name for the host instance.  This is not allowed.  Host instance should be running on server \"{0}\" but is running on server \"{1}\".",
                    hostInstance.ServerName, Environment.MachineName);
                EventLogger.LogErrorEvent(errorMessage);
                return false;
            }
            else return true;
        }

        public static string GetNQueueServiceName(HostInstance instance)
        {
            return string.Format("NQueue_{0}", instance.HostName);
        }

        public static string GetNQueueServiceDescription(HostInstance instance)
        {
            return string.Format("NQueue Processing Service : {0}", instance.HostName);
        }

        public static string GetNQueueExecutablePath(HostInstance instance)
        {
            string serviceExecutableFileName = "NQueue.Service.exe";
            //need to be able to find the service executable to install it
            if (!File.Exists(serviceExecutableFileName))
            {
                throw new FileNotFoundException(string.Format("Can't find the service executable to install.  Looking for {0} in {1}", serviceExecutableFileName, Environment.CurrentDirectory), serviceExecutableFileName);
            }
            FileInfo fiExecutable = new FileInfo(serviceExecutableFileName);
            string fullPath = fiExecutable.FullName;

            //host instance ID should be passed to the NQueue service on the command line
            return string.Format("{0} {1}", fullPath, instance.HostInstanceID);
        }

    }
}
