using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Client.Entities;
using NQueue.Client.DataServices;
using System.Data.SqlClient;
using System.Data;

namespace NQueue.Client.DataServices
{
    /// <summary>
    /// Database gateway class for work logging
    /// </summary>
    public class DBWorkLog
    {
        /// <summary>
        /// Adds a work item to the work item log
        /// </summary>
        /// <param name="connectionString">Connection string to use to log the information</param>
        /// <param name="WorkItemID">Work item ID being logged</param>
        /// <param name="logType">Type of log entry - information, warning or error</param>
        /// <param name="statusDescription">Text of the log entry</param>
        /// <param name="exc">If an error, exception message associated with the error</param>
        /// <remarks>
        /// Connection string is passed into this call as the work logger is mobile across
        /// app domains and cannot rely on having app.config entries present when being called
        /// from a job implementation.
        /// </remarks>
        public static void AddWorkLogItem(string connectionString, long WorkItemID, WorkLogTypes logType, string statusDescription, Exception exc)
        {
            string exceptionString = null;
            if (logType == WorkLogTypes.Error && exc != null)
            {
                exceptionString = string.Format("{0} -> {1}", exc.Message, exc.StackTrace);
            }
            using (SqlConnection cnn = new SqlConnection(connectionString))
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_WorkLog_AddLogItem");
                cmd.Parameters.Add("@WorkItemID", SqlDbType.BigInt).Value = WorkItemID;
                cmd.Parameters.Add("@WorkLogTypeID", SqlDbType.Int).Value = (int)logType;
                cmd.Parameters.Add("@StatusDescription", SqlDbType.NVarChar, 1000).Value = statusDescription;
                cmd.Parameters.Add("@ExceptionStackTrace", SqlDbType.NVarChar, -1).Value = exceptionString ?? DBCommon.DBNull;
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }
    }
}
