using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Client.LoggingStrategies;
using NQueue.Client.DataServices;

namespace NQueue.Client
{
    /// <summary>
    /// Execution context for the job executing in its own app domain - contains
    /// references to services the client job can use
    /// </summary>
    [Serializable]
    public class ExecutionContext
    {
        private IExecutionLoggingStrategy _loggingStrategy;
        private string _connectionString;
        private TimeSpan _timeToLive;

        public TimeSpan TimeToLive
        {
            get { return _timeToLive; }
            set { _timeToLive = value; }
        }


        /// <summary>
        /// Connection string to the batch database 
        /// </summary>
        public string ConnectionString
        {
            get { return _connectionString; }
            set { _connectionString = value; }
        }


        public ExecutionContext(IExecutionLoggingStrategy logger, TimeSpan timeToLive)
        {
            this._loggingStrategy = logger;
            this._timeToLive = timeToLive;
            //read the connection string and cache it for later. 
            //The instance will be newed up in an app domain that knows the connection string
            //but then may be serialised over to an app domain that does not
            this._connectionString = DBCommon.ReadConnectionString(DBCommon.CONNECTIONSTRING_APP_KEY);
        }

        /// <summary>
        /// Returns the event logger for the context the job is executing in
        /// </summary>
        /// <returns></returns>
        public IExecutionLoggingStrategy Logger
        {
            get { return _loggingStrategy; }
            set { _loggingStrategy = value; }
        }
        /// <summary>
        /// Returns a mutex manager instance to allow the client to use
        /// the mutex service exposed by NQueue
        /// </summary>
        /// <returns></returns>
        public MutexManager GetMutexManager()
        {
            return new MutexManager(this._connectionString);
        }
    }
}
