using System;
using System.Collections.Generic;
using System.Text;

namespace NQueue.Client.LoggingStrategies
{
    /// <summary>
    /// Interface for logging strategies.
    /// Implementors must be marked as serializable.
    /// </summary>
    public interface IExecutionLoggingStrategy
    {
        /// <summary>
        /// Writes an information item to the log
        /// </summary>
        /// <param name="description">Description of the event</param>
        void WriteInfo(string description);

        /// <summary>
        /// Writes a warning item to the log
        /// </summary>
        /// <param name="description">Description of the warning</param>
        void WriteWarning(string description);

        /// <summary>
        /// Writes an error item to the log
        /// </summary>
        /// <param name="description">Description of the error</param>
        void WriteError(string description);

        /// <summary>
        /// Writes an error item to the log
        /// </summary>
        /// <param name="description">Description of the error</param>
        /// <param name="exception">Exception associated with the error</param>
        void WriteError(string description, Exception exception);
    }
}
