using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Client.DataServices;

namespace NQueue.Client.LoggingStrategies
{
    [Serializable]
    public class NQueueDatabaseLoggingStrategy : IExecutionLoggingStrategy
    {

        private long _workItemID;
        private string _connectionString;

        public string ConnectionString
        {
            get { return _connectionString; }
            set { _connectionString = value; }
        }

        public NQueueDatabaseLoggingStrategy(long workItemID)
        {
            this._workItemID = workItemID;
            //read the connection string and cache it for later. 
            //The instance will be newed up in an app domain that knows the connection string
            //but then may be serialised over to an app domain that does not
            this._connectionString = DBCommon.ReadConnectionString(DBCommon.CONNECTIONSTRING_APP_KEY);
        }

        public long WorkItemID
        {
            get { return _workItemID; }
            set { _workItemID = value; }
        }
        public void WriteInfo(string description)
        {
            DBWorkLog.AddWorkLogItem(_connectionString, _workItemID, NQueue.Client.Entities.WorkLogTypes.Information, description, null);
        }

        public void WriteWarning(string description)
        {
            DBWorkLog.AddWorkLogItem(_connectionString, _workItemID, NQueue.Client.Entities.WorkLogTypes.Warning, description, null);
        }

        public void WriteError(string description)
        {
            EventLogger.LogErrorEvent(description);
            DBWorkLog.AddWorkLogItem(_connectionString, _workItemID, NQueue.Client.Entities.WorkLogTypes.Error, description, null);
        }

        public void WriteError(string description, Exception exception)
        {
            EventLogger.LogErrorEvent(description, exception);
            DBWorkLog.AddWorkLogItem(_connectionString, _workItemID, NQueue.Client.Entities.WorkLogTypes.Error, description, exception);
        }

    }
}
