using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using System.Data.SqlClient;
using System.Data;

namespace NQueue.DataServices
{
    /// <summary>
    /// Database gateway class for Hosts
    /// </summary>
    public class DBHost
    {
        /// <summary>
        /// Creates a new host in the database
        /// </summary>
        /// <param name="hostName"></param>
        /// <returns>Returns the ID for the new host</returns>
        public static Host InsertHost(string hostName)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_HostInsert");
                cmd.Parameters.Add("@HostName", SqlDbType.VarChar, 70).Value = hostName;
                SqlParameter paramReturn = DBCommon.ReturnValueParameter(cmd);
                cnn.Open();
                cmd.ExecuteNonQuery();
                //return value contains the newly created Host ID
                return new Host((int)paramReturn.Value, hostName);
            }
        }

        /// <summary>
        /// Deletes a host from the database
        /// </summary>
        /// <param name="hostID"></param>
        public static void DeleteHost(int hostID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_HostDelete");
                cmd.Parameters.Add("@HostID", SqlDbType.Int).Value = hostID;
                cnn.Open();
                cmd.ExecuteNonQuery();
            }
        }

        /// <summary>
        /// Returns all hosts currently defined
        /// </summary>
        /// <returns></returns>
        public static List<Host> GetAllHosts()
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_HostsSelectAll");
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                List<Host> hosts = new List<Host>();
                while (results.Read())
                {
                    hosts.Add(new Host(results.GetInt32(0), results.GetString(1)));
                }
                return hosts;
            }
        }
        /// <summary>
        /// Gets a specific host
        /// </summary>
        /// <param name="hostID">Host ID identifying the host in the database</param>
        /// <returns>Host instance representing the host</returns>
        public static Host GetHost(int hostID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_HostsSelectOne");
                cmd.Parameters.Add("@HostID", SqlDbType.Int).Value = hostID;
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                if (!results.Read())
                {
                    throw new ArgumentOutOfRangeException("hostID", "Host ID not found for host " + hostID.ToString());
                }
                else
                {
                    return new Host(results.GetInt32(0), results.GetString(1));
                }
            }
        }
    }
}
