using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using System.Data;
using System.Data.SqlClient;

namespace NQueue.DataServices
{
    public class DBWorkLogReader
    {
        /// <summary>
        /// Gets a list of all items in the queue, regardless of status and including past items,
        /// ordered in reverse order of enqueue time
        /// </summary>
        /// <param name="jobInstanceID"></param>
        /// <returns></returns>
        public static List<WorkQueueItem> GetWorkQueueForJobInstance(int jobInstanceID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_GetAllWorkItemsForJobInstance");
                cmd.Parameters.Add("@JobInstanceID", SqlDbType.Int).Value = jobInstanceID;
                List<WorkQueueItem> workQueueItems = new List<WorkQueueItem>();
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                while (results.Read())
                {
                    WorkQueueItem item = new WorkQueueItem();
                    item.WorkItemID = results.GetInt64(0);
                    item.HostInstanceID = results.IsDBNull(1) ? 0 : results.GetInt32(1);
                    item.JobInstanceID = results.GetInt32(2);
                    item.ServerAffinity = results.IsDBNull(3) ? "" : results.GetString(3);
                    item.DateEnqueued = results.GetDateTime(4);
                    item.DateToExecute = results.GetDateTime(5);
                    item.DateDequeued = DateTimeOrNull(results, 6);
                    item.DateToExpire = DateTimeOrNull(results, 7);
                    item.DateWorkCompleted = DateTimeOrNull(results, 8);
                    item.WorkItemStatus = WorkQueueItem.GetStatusFromInt(results.GetInt32(9));
                    workQueueItems.Add(item);
                }
                return workQueueItems;
            }
        }

        /// <summary>
        /// Gets all logs for a work queue item that has been dequeued
        /// </summary>
        /// <param name="workQueueItemID"></param>
        /// <returns></returns>
        public static List<WorkLogItem> GetLogsForWorkQueueItem(long workQueueItemID)
        {
            using (SqlConnection cnn = DBCommon.GetConnection())
            {
                SqlCommand cmd = DBCommon.GetCommand(cnn, "NQueue_GetWorkLogForWorkItem");
                cmd.Parameters.Add("@WorkItemID", SqlDbType.BigInt).Value = workQueueItemID;
                List<WorkLogItem> logItems = new List<WorkLogItem>();
                cnn.Open();
                SqlDataReader results = cmd.ExecuteReader();
                while (results.Read())
                {
                    WorkLogItem item = new WorkLogItem();
                    item.WorkLogItemID = results.GetGuid(0);
                    item.WorkItemID = results.GetInt64(1);
                    item.WorkLogType = WorkLogItem.IntToWorkLogType(results.GetInt32(2));
                    item.LogTime = results.GetDateTime(3);
                    item.StatusDescription = results.GetString(4);
                    item.ExceptionStackTrace = results.IsDBNull(5) ? "" : results.GetString(5);
                    logItems.Add(item);
                }
                return logItems;
            }
        }

        private static DateTime? DateTimeOrNull(SqlDataReader results, int index)
        {
            DateTime? returnValue;
            if (results.IsDBNull(index))
                returnValue = null;
            else
                returnValue = results.GetDateTime(index);
            return returnValue;
        }
    }
}
