using System;
using System.Collections.Generic;
using System.Text;

namespace NQueue.Entities
{
    /// <summary>
    /// Represents a work item dequeued from the work queue.
    /// </summary>
    public class DequeuedWorkItem
    {
        private int _jobDefinitionId;
        private int _jobInstanceID;
        private string _jobName;
        private string _jobAssemblyStrongName;
        private string _jobClassType;
        private string _jobConfigurationFile;
        private long _workItemID;
        private string _jobParameters;
        private string _jobInstanceName;
        /// <summary>
        /// Name for the job instance dequeued
        /// </summary>
        public string JobInstanceName
        {
            get { return _jobInstanceName; }
            set { _jobInstanceName = value; }
        }




        /// <summary>
        /// XML serialised object representing the parameters for the job
        /// </summary>
        public string JobParameters
        {
            get { return _jobParameters; }
            set { _jobParameters = value; }
        }


        /// <summary>
        /// Work item ID of the dequeued work item
        /// </summary>
        public long WorkItemID
        {
            get { return _workItemID; }
            set { _workItemID = value; }
        }


        /// <summary>
        /// ID of the job definition
        /// </summary>
        public int JobDefinitionID
        {
            get
            {
                return _jobDefinitionId;
            }
            set
            {
                _jobDefinitionId = value;
            }
        }

        /// <summary>
        /// Name of the job definition
        /// </summary>
        public string JobName
        {
            get
            {
                return _jobName ?? "NULL";
            }
            set
            {
                _jobName = value;  
            }
        }


        /// <summary>
        /// Strong name of the assembly containing the class implementing the job
        /// </summary>
        public string JobAssemblyStrongName
        {
            get
            {
                return _jobAssemblyStrongName;
            }
            set
            {
                _jobAssemblyStrongName = value;
            }
        }

        /// <summary>
        /// Typename of the type that implements the job
        /// </summary>
        public string JobClassType
        {
            get
            {
                return _jobClassType;
            }
            set
            {
                _jobClassType = value;
            }
        }

        /// <summary>
        /// Optional full path and filename to the configuration file to be used when the job instance executes
        /// </summary>
        public string JobConfigFile
        {
            get
            {
                return _jobConfigurationFile;
            }
            set
            {
                _jobConfigurationFile = value;
            }
        }

        /// <summary>
        /// ID of the job instance
        /// </summary>
        public int JobInstanceID
        {
            get
            {
                return _jobInstanceID;
            }
            set
            {
                _jobInstanceID = value;
            }
        }

        private JobInstanceStates _jobInstanceState;
        /// <summary>
        /// Whether the job instance is enabled, paused or disabled
        /// </summary>
        public JobInstanceStates JobInstanceState
        {
            get { return _jobInstanceState; }
            set { _jobInstanceState = value; }
        }

        private bool _isSingleton;
        /// <summary>
        /// True if the job instance should run as a singleton - ie only one instance
        /// of this job can execute at any one time
        /// </summary>
        public bool IsSingleton
        {
            get { return _isSingleton; }
            set { _isSingleton = value; }
        }

        private TimeSpan _instanceTTLTimeSpan;
        /// <summary>
        /// Timespan after which the job instance should be expired
        /// </summary>
        public TimeSpan InstanceTTLTimeSpan
        {
            get { return _instanceTTLTimeSpan; }
            set { _instanceTTLTimeSpan = value; }
        }

    }
}
