using System;
using System.Collections.Generic;
using System.Text;

namespace NQueue.Entities
{
    /// <summary>
    /// Represents a job instance - a executable instance of a job, complete with assigned parameters
    /// and a mapping to a host
    /// </summary>
    public class JobInstance
    {
        private int _jobDefinitionID;

        /// <summary>
        /// Database ID of the job definition this instance is an instance of
        /// </summary>
        public int JobDefinitionID
        {
            get { return _jobDefinitionID; }
            set { _jobDefinitionID = value; }
        }
        private string _jobName;

        /// <summary>
        /// Name of the job definition this instance is an instance of
        /// </summary>
        public string JobName
        {
            get { return _jobName; }
            set { _jobName = value; }
        }
        private int _jobInstanceID;

        /// <summary>
        /// Database ID of this job instance
        /// </summary>
        public int JobInstanceID
        {
            get { return _jobInstanceID; }
            set { _jobInstanceID = value; }
        }
        private string _jobInstanceName;
        /// <summary>
        /// Name of the job instance
        /// </summary>
        public string JobInstanceName
        {
            get { return _jobInstanceName; }
            set { _jobInstanceName = value; }
        }
        private int _hostID;

        /// <summary>
        /// Host ID of the host the instance is applied to
        /// </summary>
        public int HostID
        {
            get { return _hostID; }
            set { _hostID = value; }
        }
        private TimeSpan _instanceTTLTimeSpan;

        /// <summary>
        /// Time to live of the job - if the job is still executing or has started and
        /// failed to report completion or failure within this timespan the job will be 
        /// cancelled.
        /// </summary>
        public TimeSpan InstanceTTLTimeSpan
        {
            get { return _instanceTTLTimeSpan; }
            set { _instanceTTLTimeSpan = value; }
        }
        private string _jobParameters;

        /// <summary>
        /// Parameters for the job, (usually) expressed as the serialised XML of an object instance containing
        /// the configuration
        /// </summary>
        public string JobParameters
        {
            get { return _jobParameters; }
            set { _jobParameters = value; }
        }

        private JobInstanceStates _jobInstanceState;
        /// <summary>
        /// Whether the job instance is enabled, paused or disabled
        /// </summary>
        public JobInstanceStates JobInstanceState
        {
            get { return _jobInstanceState; }
            set { _jobInstanceState = value; }
        }

        private bool _isSingleton;
        /// <summary>
        /// True if the job instance should run as a singleton - ie only one instance
        /// of this job can execute at any one time
        /// </summary>
        public bool IsSingleton
        {
            get { return _isSingleton; }
            set { _isSingleton = value; }
        }


        public static JobInstanceStates ConvertStateIDToState(int jobInstanceStateID)
        {
            switch (jobInstanceStateID)
            {
                case 1:
                    return JobInstanceStates.Enabled;
                case 2:
                    return JobInstanceStates.Disabled;
                case 3:
                    return JobInstanceStates.Paused;
                default:
                    throw new InvalidOperationException("Unrecognised job instance state ID: " + jobInstanceStateID.ToString());
            }
        }


        public override string ToString()
        {
            return JobInstanceName;
        }

    }
}
