using System;
using System.Collections.Generic;
using System.Text;

namespace NQueue.Entities
{
    /// <summary>
    /// Represents a work queue item to be displayed on the batch admin website
    /// </summary>
    public class WorkQueueItem
    {
        private long _workItemID;

        public long WorkItemID
        {
            get { return _workItemID; }
            set { _workItemID = value; }
        }
        private int _hostInstanceID;

        public int HostInstanceID
        {
            get { return _hostInstanceID; }
            set { _hostInstanceID = value; }
        }
        private int _jobInstanceID;

        public int JobInstanceID
        {
            get { return _jobInstanceID; }
            set { _jobInstanceID = value; }
        }
        private string _serverAffinity;

        public string ServerAffinity
        {
            get { return _serverAffinity; }
            set { _serverAffinity = value; }
        }
        private DateTime _dateEnqueued;

        public DateTime DateEnqueued
        {
            get { return _dateEnqueued; }
            set { _dateEnqueued = value; }
        }
        private DateTime _dateToExecute;

        public DateTime DateToExecute
        {
            get { return _dateToExecute; }
            set { _dateToExecute = value; }
        }
        private DateTime? _dateDequeued;

        public DateTime? DateDequeued
        {
            get { return _dateDequeued; }
            set { _dateDequeued = value; }
        }
        private DateTime? _dateToExpire;

        public DateTime? DateToExpire
        {
            get { return _dateToExpire; }
            set { _dateToExpire = value; }
        }
        private DateTime? _dateWorkCompleted;

        public DateTime? DateWorkCompleted
        {
            get { return _dateWorkCompleted; }
            set { _dateWorkCompleted = value; }
        }
        private WorkItemStatus _workItemStatus;

        public WorkItemStatus WorkItemStatus
        {
            get { return _workItemStatus; }
            set { _workItemStatus = value; }
        }

        public static WorkItemStatus GetStatusFromInt(int statusID)
        {
            switch (statusID)
            {
                case 1:
                    return WorkItemStatus.New;
                case 2:
                    return WorkItemStatus.Processing;
                case 3:
                    return WorkItemStatus.Completed;
                case 4:
                    return WorkItemStatus.Failed;
                case 5:
                    return WorkItemStatus.Expired;
                case 6:
                    return WorkItemStatus.TimedOut;
                case 7:
                    return WorkItemStatus.Duplicate;
                case 8:
                    return WorkItemStatus.Disabled;
                default:
                    throw new InvalidOperationException("Did not recognised status value for int");
            }
        }
    }
    /// <summary>
    /// All possible statuses a work item can have
    /// </summary>
    public enum WorkItemStatus
    {
        /// <summary>
        /// Newly inserted
        /// </summary>
        New = 1,
        /// <summary>
        /// Currently processing
        /// </summary>
        Processing = 2,
        /// <summary>
        /// Completed successfully
        /// </summary>
        Completed = 3,
        /// <summary>
        /// Completed and failed
        /// </summary>
        Failed = 4,
        /// <summary>
        /// Item expired before being processed - not implemented yet
        /// </summary>
        Expired = 5,
        /// <summary>
        /// Item started processing but did not complete before being marked as timed out - not implemented yet
        /// </summary>
        TimedOut = 6,
        /// <summary>
        /// Item was a duplicate - processor failed to get the mutex required as another process
        /// had the mutex and was processing the work item
        /// </summary>
        Duplicate = 7,
        /// <summary>
        /// Job instance associated with the work item was disabled
        /// </summary>
        Disabled = 8
    }
}
