using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Entities;
using NQueue.DataServices;
using System.Diagnostics;
using NQueue.BusinessServices.Scheduling;
using System.Threading;
using NQueue.BusinessServices.Jobs;
using NQueue.Client.LoggingStrategies;
using System.Data.SqlClient;
using NQueue.Service.Server;
using NQueue.Client;

namespace NQueue.Service.Test
{
    class Program
    {

        private static int _hostInstanceID;
        static void Main(string[] args)
        {
            //FullTestImplementation();
            NonTestImplementation();
        }

        private static void NonTestImplementation()
        {
            Console.WriteLine("Enter host instance ID to impersonate");
            string instanceAsString = Console.ReadLine();
            int hostInstanceID = Int32.Parse(instanceAsString);
            Trace.Listeners.Add(new TextWriterTraceListener(Console.Out));
            ServiceImplementation server = new ServiceImplementation(hostInstanceID, true);
            server.StartService();
            Console.ForegroundColor = ConsoleColor.Cyan;
            Console.WriteLine("Service started, any key to end....");

            Console.ReadLine();

            Console.WriteLine("Cleaning up");
            server.StopService();
            server.Dispose();
        }

        private static void FullTestImplementation()
        {
            EventLogger.EnsureEventSource();
            string jobParams = string.Format("<ParameteredJobTypeParameters><Param1>{0}</Param1><Param2>{1}</Param2></ParameteredJobTypeParameters>", 2, 2);
            string jobAssemblyStrongName = "NQueue.UnitTests, Version=1.0.0.0, Culture=neutral, PublicKeyToken=d5f1da7c501d0b59";
            string jobClassType = "NQueue.UnitTests.ParameteredJobType";
            string jobConfigFile = @"..\..\..\NQueue.UnitTests\TestBatchJobConfiguration.dll.config";

            Host testHost = DBHost.InsertHost("TestHost " + Guid.NewGuid().ToString());
            HostInstance testHostInstance = DBHostInstance.InsertHostInstance(testHost.HostID, "Server " + Guid.NewGuid().ToString());
            JobDefinition testJobDef = DBJobDefinition.InsertJobDefinition("Job def " + Guid.NewGuid().ToString(), "JobDesc", jobAssemblyStrongName, jobClassType, jobConfigFile);
            JobInstance testJobInstance = DBJobInstance.InsertJobInstance(testJobDef.JobDefinitionID, "Job inst" + Guid.NewGuid().ToString(), testHost.HostID, new TimeSpan(1, 15, 0), jobParams, false);
            Trace.Listeners.Add(new TextWriterTraceListener(Console.Out));

            List<ScheduleType> schedules = DBScheduling.GetAllScheduleTypes();
            ScheduleType schedule = schedules[0];

            //insert a schedule
            string schedParams = "<DailyScheduleProcessorParameters><DailyProcessVariant>EveryDay</DailyProcessVariant><EveryNDaysFrequency>0</EveryNDaysFrequency><StartDateAndTime>2008-04-22T14:30:33.0000000+01:00</StartDateAndTime></DailyScheduleProcessorParameters>";
            int newScheduleID = DBJobInstance.InsertInstanceSchedule(testJobInstance.JobInstanceID, schedule.ScheduleTypeID, schedParams);
            try
            {


                _hostInstanceID = testHostInstance.HostInstanceID;

                Console.ForegroundColor = ConsoleColor.DarkBlue;
                Console.WriteLine("Started batch test processor");
                Console.ForegroundColor = ConsoleColor.Cyan;
                Trace.WriteLine("Starting threads");
                Console.ForegroundColor = ConsoleColor.Yellow;
                Trace.WriteLine("Host Instance ID is " + _hostInstanceID.ToString());

                Console.ForegroundColor = ConsoleColor.Cyan;

                System.Threading.Timer schedulingTimer = new System.Threading.Timer(new System.Threading.TimerCallback(OnSchedule), null, new TimeSpan(0, 0, 2), new TimeSpan(0, 0, 2));
                System.Threading.Timer dequeuingTimer = new System.Threading.Timer(new System.Threading.TimerCallback(OnDequeue), null, new TimeSpan(0, 0, 3), new TimeSpan(0, 0, 5));

                Console.WriteLine("Timers started, any key to end....");

                Console.ReadLine();
                Console.WriteLine("Cleaning up");


                System.Threading.WaitHandle scheduleWait = new ManualResetEvent(false);
                System.Threading.WaitHandle dequeueWait = new ManualResetEvent(false);
                schedulingTimer.Dispose(scheduleWait);
                dequeuingTimer.Dispose(dequeueWait);
                scheduleWait.WaitOne();
                dequeueWait.WaitOne();

            }
            finally
            {
                //clean up any remaining work items
                using (SqlConnection cnn = DBCommon.GetConnection())
                {
                    SqlCommand cmd = new SqlCommand();
                    cmd.Connection = cnn;
                    cmd.CommandText = "delete from WorkQueue where JobInstanceID = " + testJobInstance.JobInstanceID;
                    cmd.CommandType = System.Data.CommandType.Text;
                    cnn.Open();
                    cmd.ExecuteNonQuery();
                }
                DBJobInstance.DeleteInstanceSchedule(newScheduleID);
            }

            DBJobInstance.DeleteJobInstance(testJobInstance.JobInstanceID);
            DBJobDefinition.DeleteJobDefinition(testJobDef.JobDefinitionID);
            DBHostInstance.DeleteHostInstance(testHostInstance.HostInstanceID);
            DBHost.DeleteHost(testHost.HostID);
        }

        private static void OnSchedule(object state)
        {
            Console.WriteLine("Scheduler called");
            Scheduler.CheckAndExecuteSchedules(_hostInstanceID);
        }

        private static void OnDequeue(object state)
        {
            Console.WriteLine("Dequeuer called");
            JobDequeuer.Execute(_hostInstanceID);
        }
    }
}
