using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using NQueue.BusinessServices.System;
using NQueue.Service.Server;
using NQueue.Client;
using NQueue.Entities;
using NQueue.BusinessServices.Jobs;

namespace NQueue.Service
{
    /// <summary>
    /// Main windows service implementation for the NQueue processing service
    /// </summary>
    public partial class NQueueService : ServiceBase
    {
        HostInstance _hostInstance;
        ServiceImplementation _server;

        public NQueueService()
        {
            EventLogger.LogInformationEvent("Initialising NQueue service");
            try
            {
                InitializeComponent();
                //get the arguments specified to launch the service
                string[] imagePathArgs = Environment.GetCommandLineArgs();
                //arg[0] is the full command line, as normal - host instance ID should
                //be at arg[1]
                if (imagePathArgs.Length != 2) throw new InvalidOperationException("Service launched without the HostInstanceID parameter");
                int hostInstanceID;
                if (!int.TryParse(imagePathArgs[1], out hostInstanceID))
                    throw new InvalidOperationException("Could not parse the host instance ID from the command line parameter specified to launch the service: " + imagePathArgs[1]);

                //set the correct service name for the service
                this._hostInstance = HostInstanceManager.GetHostInstance(hostInstanceID);
                this.ServiceName = SystemManager.GetNQueueServiceName(this._hostInstance);

                EventLogger.LogInformationEvent(string.Format("NQueue service starting with parameters HostInstanceID: {0}, HostInstance name: {1}, ServiceName: {2}",
                    hostInstanceID, this._hostInstance.HostName, this.ServiceName));
            }
            catch (Exception exc)
            {
                EventLogger.LogErrorEvent("Error starting NQueue service", exc);
                throw;
            }
        }

        protected override void OnStart(string[] args)
        {
            this._server = new ServiceImplementation(this._hostInstance.HostInstanceID, false);
            this._server.StartService();
            EventLogger.LogInformationEvent("Started NQueue service: " + this.ServiceName);
        }

        protected override void OnStop()
        {
            this._server.StopService();
            EventLogger.LogInformationEvent("Stopped NQueue service: " + this.ServiceName);
        }
    }
}
