﻿using NUnit.Framework;
using System;
using System.Text;
using System.Collections.Generic;
using NQueue.BusinessServices.Jobs;
using NQueue.DataServices;
using NQueue.Entities;
using System.Xml.Serialization;
using System.IO;
using System.Diagnostics;
using System.Data.SqlClient;
using NQueue.Client.LoggingStrategies;
namespace NQueue.UnitTests
{
    /// <summary>
    ///This is a test class for NQueue.BusinessServices.Jobs.JobDequeuer and is intended
    ///to contain all NQueue.BusinessServices.Jobs.JobDequeuer Unit Tests
    ///</summary>
    [TestFixture]
    public class JobDequeuerTest
    {




        private Host _testHost;
        private HostInstance _testHostInstance;
        private JobDefinition _testJobDef;
        private JobInstance _testJobInstance;
        private DateTime _dateInPast;


        [TestFixtureSetUp]
        public void MyClassInitialize()
        {
            string jobParams = string.Format("<ParameteredJobTypeParameters><Param1>{0}</Param1><Param2>{1}</Param2></ParameteredJobTypeParameters>", 2, 2);
            string jobAssemblyStrongName = "NQueue.UnitTests, Version=1.0.0.0, Culture=neutral, PublicKeyToken=d5f1da7c501d0b59";
            string jobClassType = "NQueue.UnitTests.ParameteredJobType";
            string jobConfigFile = @"TestBatchJobConfiguration.dll.config";


            _testHost = DBHost.InsertHost("TestHost " + Guid.NewGuid().ToString());
            _testHostInstance = DBHostInstance.InsertHostInstance(_testHost.HostID, "Server " + Guid.NewGuid().ToString());
            _testJobDef = DBJobDefinition.InsertJobDefinition("Job def " + Guid.NewGuid().ToString(), "JobDesc", jobAssemblyStrongName, jobClassType, jobConfigFile);
            _testJobInstance = DBJobInstance.InsertJobInstance(_testJobDef.JobDefinitionID, "Job inst" + Guid.NewGuid().ToString(), _testHost.HostID, new TimeSpan(1, 15, 0), jobParams, false);
            _dateInPast = DateTime.Now.AddMinutes(-1);

        }


        [TestFixtureTearDown]
        public void MyClassCleanup()
        {
            DBJobInstance.DeleteJobInstance(_testJobInstance.JobInstanceID);
            DBJobDefinition.DeleteJobDefinition(_testJobDef.JobDefinitionID);
            DBHostInstance.DeleteHostInstance(_testHostInstance.HostInstanceID);
            DBHost.DeleteHost(_testHost.HostID);
        }

        /// <summary>
        ///A test for Execute ()
        ///</summary>
        [Test]
        public void ExecuteTest()
        {
            //ParameteredJobType testJob = new ParameteredJobType();
            //ParameteredJobType.ParameteredJobTypeParameters parametersObject = (ParameteredJobType.ParameteredJobTypeParameters) testJob.GetBlankParametersObject();
            //XmlSerializer ser = new XmlSerializer(testJob.ParametersType);
            //MemoryStream serialisedParameterObject = new MemoryStream();
            //ser.Serialize(serialisedParameterObject, parametersObject);
            //string serialisedAsString = Encoding.UTF8.GetString(serialisedParameterObject.ToArray());
            //Debug.WriteLine(serialisedAsString);
            try
            {
                //enqueue a job instance
                DBWorkQueue.EnqueueWorkItem(_testJobInstance.JobInstanceID, _dateInPast);
                //dequeue and run it
                JobDequeuer.Execute(_testHostInstance.HostInstanceID);

                //enqueue more job instances
                for (int i = 0; i < 6; i++)
                {
                    DBWorkQueue.EnqueueWorkItem(_testJobInstance.JobInstanceID, _dateInPast);
                }

                for (int i = 0; i < 6; i++)
                {
                    JobDequeuer.Execute(_testHostInstance.HostInstanceID);
                }

                //check they show as completed now.
                string checkQueueStatusString = string.Format("select count(*) from WorkQueue where HostInstanceID = {0} and WorkItemStatusID = 3", _testHostInstance.HostInstanceID);
                Assert.AreEqual(7, Common.ExecuteScalarInt(checkQueueStatusString));

            }
            finally
            {
                Common.ExecuteNonQuery(string.Format("delete from worklog where workitemid in (select workitemid from workqueue where jobinstanceid = {0})", _testJobInstance.JobInstanceID));
                Common.ExecuteNonQuery("delete from WorkQueue where JobInstanceID = " + _testJobInstance.JobInstanceID);
            }
        }

        [Test]
        public void ExecuteServerAffinityDequeue()
        {
            //ParameteredJobType testJob = new ParameteredJobType();
            //ParameteredJobType.ParameteredJobTypeParameters parametersObject = (ParameteredJobType.ParameteredJobTypeParameters) testJob.GetBlankParametersObject();
            //XmlSerializer ser = new XmlSerializer(testJob.ParametersType);
            //MemoryStream serialisedParameterObject = new MemoryStream();
            //ser.Serialize(serialisedParameterObject, parametersObject);
            //string serialisedAsString = Encoding.UTF8.GetString(serialisedParameterObject.ToArray());
            //Debug.WriteLine(serialisedAsString);
            try
            {

                //enqueue some job instances with no affinity
                for (int i = 0; i < 6; i++)
                {
                    DBWorkQueue.EnqueueWorkItem(_testJobInstance.JobInstanceID, _dateInPast);
                }
                //enqueue some job instances with affinity to this server
                for (int i = 0; i < 6; i++)
                {
                    DBWorkQueue.EnqueueWorkItem(_testJobInstance.JobInstanceID, 
                        System.Environment.MachineName ,_dateInPast);
                }
                //enqueue some job instances with a fictional servername
                for (int i = 0; i < 6; i++)
                {
                    DBWorkQueue.EnqueueWorkItem(_testJobInstance.JobInstanceID, 
                        "NonExistantServername", _dateInPast);
                }
                
                //try and dequeue them all
                JobDequeuer.Execute(_testHostInstance.HostInstanceID);
 
                //check the non-affinity and correct affinity show as completed now.
                string checkQueueStatusStringComplete = string.Format("select count(*) from WorkQueue where JobInstanceID = {0} and WorkItemStatusID = 3", _testJobInstance.JobInstanceID);
                Assert.AreEqual(12, Common.ExecuteScalarInt(checkQueueStatusStringComplete));

                //check the wrong affinity ones are still showing as new
                string checkQueueStatusStringNew = string.Format("select count(*) from WorkQueue where JobInstanceID = {0} and WorkItemStatusID = 1", _testJobInstance.JobInstanceID);
                Assert.AreEqual(6, Common.ExecuteScalarInt(checkQueueStatusStringNew));

            }
            finally
            {
                Common.ExecuteNonQuery(string.Format("delete from worklog where workitemid in (select workitemid from workqueue where jobinstanceid = {0})", _testJobInstance.JobInstanceID));
                Common.ExecuteNonQuery("delete from WorkQueue where JobInstanceID = " + _testJobInstance.JobInstanceID);
            }
        }
    }


}
