﻿using System;
using System.Text;
using System.Collections.Generic;
using NQueue.Entities;
using NQueue.DataServices;
using NUnit.Framework;

namespace NQueue.UnitTests.DataServices
{
    [TestFixture]
    public class DBHostInstanceTest
    {



        /// <summary>
        ///A test for DeleteHostInstance (int)
        ///</summary>
        [Test]
        public void DeleteHostInstanceTest()
        {
            string newHostName = "TestHost " + Guid.NewGuid().ToString();
            string newInstanceServerName = "Server " + Guid.NewGuid().ToString();
            int countBefore = DBHostInstance.GetAllHostInstances().Count;
            Host newHost = DBHost.InsertHost(newHostName);
            try
            {
                HostInstance newHostInstance = DBHostInstance.InsertHostInstance(newHost.HostID, newInstanceServerName);
                try
                {
                    DBHostInstance.DeleteHostInstance(newHostInstance.HostInstanceID);
                    int countAfter = DBHostInstance.GetAllHostInstances().Count;
                    Assert.AreEqual(countBefore, countAfter);
                }
                finally { DBHostInstance.DeleteHostInstance(newHostInstance.HostInstanceID); }
            }
            finally {DBHost.DeleteHost(newHost.HostID);}
        }

        /// <summary>
        ///A test for GetHostInstance (int)
        ///</summary>
        [Test]
        public void GetHostInstanceTest()
        {
            string newHostName = "TestHost " + Guid.NewGuid().ToString();
            string newInstanceServerName = "Server " + Guid.NewGuid().ToString();
            Host newHost = DBHost.InsertHost(newHostName);
            try
            {
                HostInstance newHostInstance = DBHostInstance.InsertHostInstance(newHost.HostID, newInstanceServerName);
                try
                {
                    //retrieve the host instance by ID
                    HostInstance instanceRetrievedByID = DBHostInstance.GetHostInstance(newHostInstance.HostInstanceID);
                    Assert.AreEqual(newHostInstance.HostInstanceID, instanceRetrievedByID.HostInstanceID);
                    Assert.AreEqual(newHostName, instanceRetrievedByID.HostName);
                    Assert.AreEqual(newInstanceServerName, instanceRetrievedByID.ServerName);
                    Assert.AreEqual(newHost.HostID, instanceRetrievedByID.HostID);

                    //retrieve the instance by searching on the host
                    List<HostInstance> searchedInstances = DBHostInstance.GetAllHostInstances(newHost.HostID);
                    Assert.AreEqual(1, searchedInstances.Count);
                    HostInstance foundInstance = searchedInstances[0];
                    Assert.AreEqual(newHostInstance.HostInstanceID, foundInstance.HostInstanceID);
                    Assert.AreEqual(newInstanceServerName, foundInstance.ServerName);
                    Assert.AreEqual(newHost.HostID, foundInstance.HostID);

                }
                finally { DBHostInstance.DeleteHostInstance(newHostInstance.HostInstanceID); }
            }
            finally { DBHost.DeleteHost(newHost.HostID); }
        }

        /// <summary>
        ///A test for InsertHostInstance (int, string)
        ///</summary>
        [Test]
        public void InsertHostInstanceTest()
        {
            string newHostName = "TestHost " + Guid.NewGuid().ToString();
            string newInstanceServerName = "Server " + Guid.NewGuid().ToString();
            Host newHost = DBHost.InsertHost(newHostName);
            try
            {
                HostInstance newHostInstance = DBHostInstance.InsertHostInstance(newHost.HostID, newInstanceServerName);
                try
                {
                    Assert.AreEqual(newHost.HostID, newHostInstance.HostID);
                    Assert.AreEqual(newInstanceServerName, newHostInstance.ServerName);
                }
                finally { DBHostInstance.DeleteHostInstance(newHostInstance.HostInstanceID); }
            }
            finally { DBHost.DeleteHost(newHost.HostID); }
        }

    }


}
