﻿using NUnit.Framework;
using System;
using System.Text;
using System.Collections.Generic;
using NQueue.Entities;
using NQueue.DataServices;
namespace NQueue.UnitTests.DataServices
{
    /// <summary>
    ///This is a test class for NQueue.DataServices.DBHost and is intended
    ///to contain all NQueue.DataServices.DBHost Unit Tests
    ///</summary>
    [TestFixture]
    public class DBHostTest
    {




        /// <summary>
        ///A test for DeleteHost (int)
        ///</summary>
        [Test]
        public void DeleteHostTest()
        {
            List<Host> allHosts = DBHost.GetAllHosts();
            int beforeCount = allHosts.Count;
            Host newHost = DBHost.InsertHost("Test Host " + Guid.NewGuid().ToString());
            DBHost.DeleteHost(newHost.HostID);
            allHosts = DBHost.GetAllHosts();
            int afterCount = allHosts.Count;

            Assert.AreEqual(beforeCount, afterCount);
        }



        /// <summary>
        ///A test for GetHost (int)
        ///</summary>
        [Test]
        public void GetHostTest()
        {
            string newHostName = "Test Host " + Guid.NewGuid().ToString();
            Host newHost = DBHost.InsertHost(newHostName);
            try
            {
                //retrieve inserted host
                Host retrievedHost = DBHost.GetHost(newHost.HostID);
                Assert.AreEqual(newHost.HostID, retrievedHost.HostID);
                Assert.AreEqual(newHostName, retrievedHost.HostName);
            }
            finally
            {
                DBHost.DeleteHost(newHost.HostID);
            }
        }

        /// <summary>
        ///A test for InsertHost (Host)
        ///</summary>
        [Test]
        public void InsertHostTest()
        {
            string newHostName = "Test Host " + Guid.NewGuid().ToString();
            Host newHost = DBHost.InsertHost(newHostName);
            DBHost.DeleteHost(newHost.HostID);
            Assert.AreEqual(newHostName, newHost.HostName);
        }

    }


}
