﻿using NUnit.Framework;
using System;
using System.Text;
using System.Collections.Generic;
using NQueue.Entities;
using NQueue.DataServices;
namespace NQueue.UnitTests.DataServices
{
    /// <summary>
    ///This is a test class for NQueue.DataServices.DBJobDefinition and is intended
    ///to contain all NQueue.DataServices.DBJobDefinition Unit Tests
    ///</summary>
    [TestFixture]
    public class DBJobDefinitionTest
    {




        private Host _testHost;
        private HostInstance _testHostInstance;

        [TestFixtureSetUp]
        public void MyClassInitialize()
        {
            string newHostName = "TestHost " + Guid.NewGuid().ToString();
            string newInstanceServerName = "Server " + Guid.NewGuid().ToString();
            _testHost = DBHost.InsertHost(newHostName);
            _testHostInstance = DBHostInstance.InsertHostInstance(_testHost.HostID, newInstanceServerName);
        }
        
        
        [TestFixtureTearDown]
        public void MyClassCleanup()
        {
            DBHostInstance.DeleteHostInstance(_testHostInstance.HostInstanceID);
            DBHost.DeleteHost(_testHost.HostID);
        }

        /// <summary>
        ///A test for DeleteJobDefinition (int)
        ///</summary>
        [Test]
        public void DeleteJobDefinitionTest()
        {
            string newJobDefName = "Job def " + Guid.NewGuid().ToString();
            int countBefore = DBJobDefinition.GetAllJobDefinitions().Count;
            JobDefinition newJobDef = DBJobDefinition.InsertJobDefinition(newJobDefName, "JobDesc", "JobAsmSN", "JobCT", "JobConfigFile");
            DBJobDefinition.DeleteJobDefinition(newJobDef.JobDefinitionID);
            int countAfter = DBJobDefinition.GetAllJobDefinitions().Count;
            Assert.AreEqual(countBefore, countAfter);
        }

        /// <summary>
        ///A test for GetJobDefinition (int)
        ///</summary>
        [Test]
        public void GetJobDefinitionTest()
        {
            string newJobDefName = "Job def " + Guid.NewGuid().ToString();
            string jobDesc = "JobDesc";
            string jobAsmName = "JobAsmSN";
            string jobClassType = "JobCT";
            string jobConfigFile = "JobConfigFile";

            JobDefinition newJobDef = DBJobDefinition.InsertJobDefinition(newJobDefName, jobDesc, 
                jobAsmName, jobClassType, jobConfigFile);
            AssertJobDetails(newJobDefName, jobDesc, jobAsmName, jobClassType, jobConfigFile, newJobDef);

            try
            {
                JobDefinition retrievedJobDef = DBJobDefinition.GetJobDefinition(newJobDef.JobDefinitionID);
                Assert.AreEqual(newJobDef.JobDefinitionID, retrievedJobDef.JobDefinitionID);
                AssertJobDetails(newJobDefName, jobDesc, jobAsmName, jobClassType, jobConfigFile, retrievedJobDef);
            }
            finally
            {
                DBJobDefinition.DeleteJobDefinition(newJobDef.JobDefinitionID);
            }
        }

        private void AssertJobDetails(string jobName, string jobDesc, string jobAsmName, string jobClassType, string jobConfigFile, JobDefinition newJobDef)
        {
            Assert.AreEqual(jobName, newJobDef.JobName);
            Assert.AreEqual(jobDesc, newJobDef.JobDescription);
            Assert.AreEqual(jobAsmName, newJobDef.JobAssemblyStrongName);
            Assert.AreEqual(jobClassType, newJobDef.JobClassType);
            Assert.AreEqual(jobConfigFile, newJobDef.JobConfigFile);
        }

    }


}
