using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using NQueue.Client.DataServices;

namespace NQueue.UnitTests.DataServices
{
    [TestFixture]
    public class DBMutexTest
    {
        private readonly string _machineName = Environment.MachineName;
        private readonly string _connectionString = DBCommon.ReadConnectionString(DBCommon.CONNECTIONSTRING_APP_KEY);


        [Test]
        public void GetAndDeleteMutex()
        {
            string mutexName = Guid.NewGuid().ToString();
            string sqlCountMutex = string.Format("select count(*) from Mutex where MutexName = '{0}'", mutexName);
            Assert.AreEqual(0, Common.ExecuteScalarInt(sqlCountMutex));
            int? mutexID;
            //can get the mutex
            Assert.IsTrue(DBMutex.ObtainMutex(_connectionString, mutexName, _machineName,
                new TimeSpan(0, 1, 0), out mutexID));
            int? mutexID2;
            //can't get it again because held at mutexid1
            Assert.IsFalse(DBMutex.ObtainMutex(_connectionString, mutexName, _machineName,
                new TimeSpan(0, 1, 0), out mutexID2));
            Assert.AreEqual(1, Common.ExecuteScalarInt(sqlCountMutex));
            DBMutex.DeleteMutex(_connectionString, mutexID.Value, _machineName);
        }

        [Test]
        public void GetAndReleaseMutex()
        {
            string mutexName = Guid.NewGuid().ToString();
            string sqlCountMutexHeld = string.Format("select count(*) from Mutex where MutexName = '{0}' and ServerWithLock is not null", mutexName);
            string sqlCountMutexNotHeld = string.Format("select count(*) from Mutex where MutexName = '{0}' and ServerWithLock is null", mutexName);
            Assert.AreEqual(0, Common.ExecuteScalarInt(sqlCountMutexHeld));
            int? mutexID;
            Assert.IsTrue(DBMutex.ObtainMutex(_connectionString, mutexName, _machineName,
                new TimeSpan(0, 1, 0), out mutexID));
            Assert.AreEqual(1, Common.ExecuteScalarInt(sqlCountMutexHeld));
            DBMutex.ReleaseMutex(_connectionString, mutexID.Value, _machineName);
            Assert.AreEqual(1, Common.ExecuteScalarInt(sqlCountMutexNotHeld));
            Assert.AreEqual(0, Common.ExecuteScalarInt(sqlCountMutexHeld));
            Common.ExecuteNonQuery(string.Format("delete Mutex where MutexName = '{0}'", mutexName));
        }


    }


}
