﻿using NUnit.Framework;
using System;
using System.Text;
using System.Collections.Generic;
using NQueue.Entities;
using NQueue.DataServices;
using NQueue.BusinessServices.Scheduling;
namespace NQueue.UnitTests
{
    /// <summary>
    ///This is a test class for NQueue.BusinessServices.Scheduling.Scheduler and is intended
    ///to contain all NQueue.BusinessServices.Scheduling.Scheduler Unit Tests
    ///</summary>
    [TestFixture]
    public class SchedulerTest
    {



        private Host _testHost;
        private HostInstance _testHostInstance;
        private JobDefinition _testJobDef;
        private JobInstance _testJobInstance;

        [TestFixtureSetUp]
        public void MyClassInitialize()
        {
            string jobParams = string.Format("<ParameteredJobTypeParameters><Param1>{0}</Param1><Param2>{1}</Param2></ParameteredJobTypeParameters>", 2, 2);
            string jobAssemblyStrongName = "NQueue.UnitTests, Version=1.0.0.0, Culture=neutral, PublicKeyToken=d5f1da7c501d0b59";
            string jobClassType = "NQueue.UnitTests.ParameteredJobType";
            string jobConfigFile = @"TestBatchJobConfiguration.dll.config";


            _testHost = DBHost.InsertHost("TestHost " + Guid.NewGuid().ToString());
            _testHostInstance = DBHostInstance.InsertHostInstance(_testHost.HostID, "Server " + Guid.NewGuid().ToString());
            _testJobDef = DBJobDefinition.InsertJobDefinition("Job def " + Guid.NewGuid().ToString(), "JobDesc", jobAssemblyStrongName, jobClassType, jobConfigFile);
            _testJobInstance = DBJobInstance.InsertJobInstance(_testJobDef.JobDefinitionID, "Job inst" + Guid.NewGuid().ToString(), _testHost.HostID, new TimeSpan(1, 15, 0), jobParams, false);

        }


        [TestFixtureTearDown]
        public void MyClassCleanup()
        {
            DBJobInstance.DeleteJobInstance(_testJobInstance.JobInstanceID);
            DBJobDefinition.DeleteJobDefinition(_testJobDef.JobDefinitionID);
            DBHostInstance.DeleteHostInstance(_testHostInstance.HostInstanceID);
            DBHost.DeleteHost(_testHost.HostID);
        }

        /// <summary>
        ///A test for CheckAndExecuteSchedules (int)
        ///</summary>
        [Test]
        public void CheckAndExecuteSchedulesTest()
        {
            //DailyScheduleProcessorParameters dailyProcessorParams = new DailyScheduleProcessorParameters();
            //dailyProcessorParams.DailyProcessVariant = DailyScheduleProcessorParameters.DayVariant.EveryDay;
            //dailyProcessorParams.StartDateAndTime = DateTime.Now.AddHours(-1);
            //XmlSerializer ser = new XmlSerializer(typeof(DailyScheduleProcessorParameters));
            //MemoryStream serialisedObject = new MemoryStream();
            //ser.Serialize(serialisedObject, dailyProcessorParams);
            //string serialisedAsString = Encoding.UTF8.GetString(serialisedObject.ToArray());
            //Debug.WriteLine(serialisedAsString);

            List<ScheduleType> schedules = DBScheduling.GetAllScheduleTypes();
            Assert.IsTrue(schedules.Count > 0);

            //work with the first schedule item (these are seed data)
            ScheduleType schedule = schedules[0];
            Assert.IsNull(schedule.ProcessingHostInstanceID);

             //insert a schedule
            string schedParams = "<DailyScheduleProcessorParameters><DailyProcessVariant>EveryDay</DailyProcessVariant><EveryNDaysFrequency>0</EveryNDaysFrequency><StartDateAndTime>2008-04-22T00:30:33.0000000+01:00</StartDateAndTime></DailyScheduleProcessorParameters>";
            int newScheduleID = DBJobInstance.InsertInstanceSchedule(_testJobInstance.JobInstanceID, schedule.ScheduleTypeID, schedParams);
            try
            {
                //manually tweak the "last processed" date to be before today so that we'll have some work to do
                string sqlTweak = string.Format("update ScheduleTypes set LastProcessed = '{0}' where ScheduleTypeID = {1}", DateTime.Now.AddDays(-1).ToString("yyyy-MM-dd hh:mm:ss"), schedule.ScheduleTypeID);
                Common.ExecuteNonQuery(sqlTweak);

                while (DBWorkQueue.DequeueWorkItem(_testHostInstance.HostInstanceID) != null) { ; }
                Scheduler.CheckAndExecuteSchedules(_testHostInstance.HostInstanceID);
                //should have enqueued 1 work item
                DequeuedWorkItem dqitem = DBWorkQueue.DequeueWorkItem(_testHostInstance.HostInstanceID);
                Assert.IsNotNull(dqitem);
                Assert.AreEqual(_testJobInstance.JobInstanceID, dqitem.JobInstanceID);
            }
            finally
            {
                DBJobInstance.DeleteInstanceSchedule(newScheduleID);
            }


            ////dequeue everything from the queue
            //while (DBWorkQueue.DequeueWorkItem(_testHostInstance.HostInstanceID) != null) { ; }
            ////process the schedule
            //Assert.IsTrue(processor.ProcessSchedule(schedule.LastProcessed));
        }

        private JobInstanceSchedule GetInsertedInstanceSchedule()
        {
            List<JobInstanceSchedule> instanceSchedules = DBJobInstance.GetJobInstanceSchedulesForJobInstance(_testJobInstance.JobInstanceID);
            Assert.AreEqual(1, instanceSchedules.Count);
            JobInstanceSchedule instanceSchedule = instanceSchedules[0];
            return instanceSchedule;
        }

    }


}
