using System;
using System.Collections.Generic;
using System.Text;
using NQueue.Client;
using System.Configuration;
using System.Diagnostics;
using NQueue.DataServices;
using NQueue.Client.LoggingStrategies;

namespace NQueue.UnitTests
{
    public class TestJobType : Job
    {
        protected override ExecutionResults ExecuteJobImplementation(ExecutionContext context)
        {
            return ExecutionResults.Success;
        }

        public override Type ParametersType
        {
            get { return typeof(string); }
        }



        protected override string JobTypeName
        {
            get { return "TestJobType"; }
        }
    }
    public class TestJobTypeConfigFile : Job
    {
        protected override ExecutionResults ExecuteJobImplementation(ExecutionContext context)
        {
            EventLogger.LogInformationEvent("Running TestJobTypeConfigFile job");
            AppSettingsReader settingsReader = new AppSettingsReader();
            string settingValue = (string) settingsReader.GetValue("TestAppSetting", typeof(System.String));
            if (settingValue == "valueResult")
                return ExecutionResults.Success;
            else
                return ExecutionResults.Failure;
        }

        public override Type ParametersType
        {
            get { return typeof(string); }
        }
        protected override string JobTypeName
        {
            get { return "TestJobTypeConfigFile"; }
        }
    }

    public class VerboseJobType : Job
    {
        protected override ExecutionResults ExecuteJobImplementation(ExecutionContext context)
        {
            IExecutionLoggingStrategy logger = context.Logger;
            logger.WriteInfo("Started verbose job");
            logger.WriteInfo("Checking parameters type");
            if (!(ParametersObject is ParameteredJobTypeParameters))
            {
                throw new InvalidOperationException("Parameters object was not of type ParameteredJobTypeParameters");
            }
            logger.WriteWarning("Having a sleep");
            System.Threading.Thread.Sleep(1000);
            logger.WriteInfo("Slept");
            ParameteredJobTypeParameters jobParams = (ParameteredJobTypeParameters)ParametersObject;
            if (jobParams.Param1 == jobParams.Param2)
            {
                logger.WriteInfo("About to return success");
                return ExecutionResults.Success;
            }
            else
            {
                logger.WriteError("Failure!");
                throw new InvalidOperationException("ERROR exception message");
            }
        }

        protected override string JobTypeName
        {
            get { return "Verbose job type";  }
        }

        public override Type ParametersType
        {
            get { return typeof(ParameteredJobTypeParameters); }
        }
    }

    public class ParameteredJobType : Job
    {

        public override Type ParametersType
        {
            get { return typeof(ParameteredJobTypeParameters); }
        }

        protected override ExecutionResults ExecuteJobImplementation(ExecutionContext context)
        {
            if (!(ParametersObject is ParameteredJobTypeParameters))
            {
                throw new InvalidOperationException("Parameters object was not of type ParameteredJobTypeParameters");
            }
            
            ParameteredJobTypeParameters jobParams = (ParameteredJobTypeParameters) ParametersObject;
            if (jobParams.Param1 == jobParams.Param2)
                return ExecutionResults.Success;
            else
                return ExecutionResults.Failure;
        }

        protected override string JobTypeName
        {
            get { return "ParameteredJobType"; }
        }



    }

    [Serializable]
    public class ParameteredJobTypeParameters
    {
        private int _param1;

        public int Param1
        {
            get { return _param1; }
            set { _param1 = value; }
        }
        private int _param2;

        public int Param2
        {
            get { return _param2; }
            set { _param2 = value; }
        }
    }

    public class ManyParametersJobType : Job
    {

        public override Type ParametersType
        {
            get { return typeof(ManyParameteredJobTypeParameters); }
        }
        protected override string JobTypeName
        {
            get { return "ManyParametersJobType"; }
        }

        protected override ExecutionResults ExecuteJobImplementation(ExecutionContext context)
        {
            if (!(ParametersObject is ManyParameteredJobTypeParameters))
            {
                throw new InvalidOperationException("Parameters object was not of type ManyParameteredJobTypeParameters");
            }

            ManyParameteredJobTypeParameters jobParams = (ManyParameteredJobTypeParameters)ParametersObject;
            if (jobParams.Param1 == jobParams.Param2)
                return ExecutionResults.Success;
            else
                return ExecutionResults.Failure;

        }

        [Serializable]
        public class ManyParameteredJobTypeParameters
        {
            private int _param1;

            public int Param1
            {
                get { return _param1; }
                set { _param1 = value; }
            }
            private int _param2;

            public int Param2
            {
                get { return _param2; }
                set { _param2 = value; }
            }

            private string _stringParam;

            public string StringParam
            {
                get { return _stringParam; }
                set { _stringParam = value; }
            }
            private bool _boolParam;

            public bool BoolParam
            {
                get { return _boolParam; }
                set { _boolParam = value; }
            }
            private DateTime _dtParam;

            public DateTime DtParam
            {
                get { return _dtParam; }
                set { _dtParam = value; }
            }
            private TimeSpan _timespanParam;

            public TimeSpan TimespanParam
            {
                get { return _timespanParam; }
                set { _timespanParam = value; }
            }
            private char _charParam;

            public char CharParam
            {
                get { return _charParam; }
                set { _charParam = value; }
            }
            private byte _byteParam;

            public byte ByteParam
            {
                get { return _byteParam; }
                set { _byteParam = value; }
            }
            private double _dblParam;

            public double DblParam
            {
                get { return _dblParam; }
                set { _dblParam = value; }
            }
            private decimal _decParam;

            public decimal DecParam
            {
                get { return _decParam; }
                set { _decParam = value; }
            }

            private string[] _stringArrayParam;

            public string[] StringArrayParam
            {
                get { return _stringArrayParam; }
                set { _stringArrayParam = value; }
            }





        }


    }
}
