using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace NQueue
{
    public partial class BaseEntityControl : UserControl
    {
        public event EventHandler AddCompleted;
        public event EventHandler DeleteCompleted;
        public delegate void RefreshRequired();

        private bool _newItem;
        private RefreshRequired _refreshCallback;

        public bool NewItem
        {
            get { return _newItem; }
            set { _newItem = value; }
        }
        public BaseEntityControl() { ;}
        public BaseEntityControl(bool newItem, RefreshRequired refreshCallback)
        {
            InitializeComponent();
            this._newItem = newItem;
            this._refreshCallback = refreshCallback;
            btnAdd.Enabled = newItem;
            btnDelete.Enabled = !newItem;
        }

        private object _controllableItem;

        public object ControllableItem
        {
            get { return _controllableItem; }
            set { _controllableItem = value; }
        }

        public void SetItem(object controllableItem)
        {
            this._controllableItem = controllableItem;
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            Add();
            _refreshCallback();
            if (AddCompleted != null) AddCompleted(this, new EventArgs());
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            Delete();
            _refreshCallback();
            if (DeleteCompleted != null) DeleteCompleted(this, new EventArgs());
        }

        protected virtual void Add() { ;}
        protected virtual void Delete() { ;}
    }
}
