using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NQueue.Entities;
using NQueue.BusinessServices.Jobs;

namespace NQueue
{
    public partial class HostControl : NQueue.BaseEntityControl
    {
        private Host _host;

        public HostControl(bool newItem, RefreshRequired refreshRequired) : base(newItem, refreshRequired)
        {
            InitializeComponent();
        }


        private void HostControl_Load(object sender, EventArgs e)
        {
            _host = (Host)ControllableItem;
            if (!NewItem)
            {
                txtHost.Text = _host.HostName;
                lblHostID.Text = _host.HostID.ToString();
            }
            txtHost.ReadOnly = !NewItem;
        }


        protected override void Delete()
        {
            if (MessageBox.Show("Delete host " + _host.HostName + ".  Are you sure?", "Delete Host", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                HostManager hostManager = new HostManager(_host);
                List<HostInstance> instances = hostManager.GetHostInstances();
                if (instances.Count > 0)
                {
                    StringBuilder instancesMessage = new StringBuilder("Cannot delete host as one or more instances reference the host\n");
                    foreach (HostInstance instance in instances)
                    {
                        instancesMessage.AppendFormat("HostName: {0}, Server name: {1}\n", instance.HostName, instance.ServerName);
                    }
                    MessageBox.Show(instancesMessage.ToString(), "Cannot delete host", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    return;
                }
                else
                {
                    hostManager.DeleteHost();
                }
            }
        }

        protected override void Add()
        {
            HostManager.AddHost(txtHost.Text);
        }
    }
}

