using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NQueue.Entities;
using NQueue.BusinessServices.Jobs;
using NQueue.Client;

namespace NQueue
{
    public partial class JobDefinitionControl : NQueue.BaseEntityControl
    {
        public JobDefinitionControl(bool newItem, RefreshRequired refreshRequired)
            : base(newItem, refreshRequired)
        {
            InitializeComponent();
        }

        protected override void Add()
        {
            JobManager.AddJobDefinition(txtJobName.Text, txtJobDescription.Text, _jobAssembly, _jobClassType, txtConfigFile.Text);
        }

        protected override void Delete()
        {
            if (MessageBox.Show("Delete Job Definition" + _jobDefinition.JobName + ".  Are you sure?", "Delete Job Definition", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                //see if there are any associated job instances before deleting the definition
                List<JobInstance> instances = JobManager.GetAllJobInstances(_jobDefinition.JobDefinitionID);
                if (instances.Count > 0)
                {
                    StringBuilder errorMessage = new StringBuilder("Cannot delete the job as one or more job instances exist for the job definition\n");
                    foreach (JobInstance instance in instances)
                    {
                        errorMessage.AppendFormat("{0}\n", instance.JobInstanceName);
                    }
                    MessageBox.Show(errorMessage.ToString(), "Cannot delete job definition", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                }
                else
                {
                    JobManager.DeleteJobDefinition(_jobDefinition);
                }
            }
        }

        private JobDefinition _jobDefinition;
        private string _jobAssembly;
        private string _jobClassType;

        private void JobDefinitionControl_Load(object sender, EventArgs e)
        {
            if (!NewItem)
            {
                _jobDefinition = (JobDefinition)ControllableItem;
                
                lblJobDefinitionID.Text = _jobDefinition.JobDefinitionID.ToString();
                txtJobName.Text = _jobDefinition.JobName;
                txtJobDescription.Text = _jobDefinition.JobDescription;
                txtAssembly.Text = string.Format("{0} -> {1}", _jobDefinition.JobAssemblyStrongName, _jobDefinition.JobClassType);
                txtConfigFile.Text = _jobDefinition.JobConfigFile;
            }
            else
            {
                txtJobName.ReadOnly = false;
                txtJobDescription.ReadOnly = false;
                btnSelectAssembly.Enabled = true;
                btnConfigSelect.Enabled = true;
            }

        }

        private void btnSelectAssembly_Click(object sender, EventArgs e)
        {
            TypeBrowserForm typeBrowser = new TypeBrowserForm(typeof(Job));
            typeBrowser.TypeSelected += new TypeBrowser.TypeSelectedDelegate(typeBrowser_TypeSelected);
            typeBrowser.ShowDialog();
        }

        void typeBrowser_TypeSelected(string assemblyFullName, string typeFullName)
        {
            this._jobAssembly = assemblyFullName;
            this._jobClassType = typeFullName;
            txtAssembly.Text = string.Format("{0} -> {1}", assemblyFullName, typeFullName);
        }

        private void btnConfigSelect_Click(object sender, EventArgs e)
        {
            if (openConfigFile.ShowDialog() == DialogResult.OK)
            {
                txtConfigFile.Text = openConfigFile.FileName;
            }
        }

    }
}

