using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NQueue.Entities;
using NQueue.BusinessServices.Jobs;
using NQueue.Client;

namespace NQueue
{
    public partial class JobInstanceControl : NQueue.BaseEntityControl
    {
        public JobInstanceControl(bool newItem, RefreshRequired refreshRequired)
            : base(newItem, refreshRequired)
        {
            InitializeComponent();
        }

        protected override void Add()
        {
            JobManager.InsertJobInstance(((JobDefinition)cboJobs.SelectedItem).JobDefinitionID,
                txtInstanceName.Text, ((Host)cboHosts.SelectedItem).HostID, GetTTLTimeSpan(), 
                instanceProperties.SelectedObject, chkSingleton.Checked);
        }

        private TimeSpan GetTTLTimeSpan()
        {
            try
            {
                return new TimeSpan(Int32.Parse(txtTTL.Text.Substring(0, 2)),
                    Int32.Parse(txtTTL.Text.Substring(3, 2)),
                    Int32.Parse(txtTTL.Text.Substring(6, 2)));
            }
            catch (FormatException excForm)
            {
                throw new FormatException("TTL was in the wrong format: " + txtTTL.Text, excForm);
            }
            catch (Exception exc)
            {
                throw new FormatException("Error converting TTL to a timespan: " + txtTTL.Text, exc);
            }
        }

        protected override void Delete()
        {
            if (MessageBox.Show("Delete Job Instance " + _jobInstance.JobInstanceName + ".  Are you sure?", "Delete Job Instance", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                List<JobInstanceSchedule> schedules = JobManager.GetSchedulesForInstance(_jobInstance);
                if (schedules.Count > 0)
                {
                    StringBuilder errorMessage = new StringBuilder("Cannot delete job instance as one or more schedules are defined for the instance:\n");
                    foreach (JobInstanceSchedule schedule in schedules)
                    {
                        errorMessage.AppendFormat("{0}\n", schedule.ScheduleTypeName);
                    }
                    MessageBox.Show(errorMessage.ToString(), "Cannot delete job instance", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                }
                else
                {
                    JobManager.DeleteJobInstance(_jobInstance);
                }
            }
        }

        JobInstance _jobInstance;

        private void JobInstanceControl_Load(object sender, EventArgs e)
        {
            cboHosts.DataSource = HostManager.GetAllHosts();
            cboJobs.DataSource = JobManager.GetAllJobDefinitions();

            if (!NewItem)
            {
                _jobInstance = (JobInstance)ControllableItem;
                
                lblJobInstanceID.Text = _jobInstance.JobInstanceID.ToString();
                foreach (Host host in cboHosts.Items)
                {
                    if (host.HostID == _jobInstance.HostID)
                    {
                        cboHosts.SelectedItem = host;
                    }
                }
                foreach (JobDefinition job in cboJobs.Items)
                {
                    if (job.JobDefinitionID == _jobInstance.JobDefinitionID)
                    {
                        cboJobs.SelectedItem = job;
                    }
                }
                txtTTL.Text = string.Format("{0:00}:{1:00}:{2:00}", _jobInstance.InstanceTTLTimeSpan.Hours,
                    _jobInstance.InstanceTTLTimeSpan.Minutes,
                    _jobInstance.InstanceTTLTimeSpan.Seconds);
                Job targetJobObject = JobManager.GetJobObject(_jobInstance.JobDefinitionID);
                instanceProperties.SelectedObject = targetJobObject.GetDeserialisedParametersObject(_jobInstance.JobParameters);
                txtInstanceName.Text = _jobInstance.JobInstanceName;
                chkSingleton.Checked = _jobInstance.IsSingleton;
            }
            else
            {
                cboHosts.Enabled = true;
                cboJobs.Enabled = true;
                txtInstanceName.ReadOnly = false;
                instanceProperties.Enabled = true;
                txtTTL.ReadOnly = false;
                chkSingleton.Enabled = true;
                btnExecute.Enabled = false;
                
            }
        }

        private void UpdatePropertyGridWithBlankProperties(int newjobDefintionID)
        {
            Job targetJobObject = JobManager.GetJobObject(newjobDefintionID);
            instanceProperties.SelectedObject = targetJobObject.GetBlankParametersObject();
        }

        private void cboJobs_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdatePropertyGridWithBlankProperties(((JobDefinition)cboJobs.SelectedItem).JobDefinitionID);
        }

        private void btnExecute_Click(object sender, EventArgs e)
        {
            JobManager.EnqueueJobInstanceNow(_jobInstance.JobInstanceID);
            MessageBox.Show("Job instance enqueued for immediate execution", "Job Instance Execute", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }



    }
}

