using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NQueue.Entities;
using NQueue.BusinessServices.Jobs;
using NQueue.BusinessServices.System;
using NQueue.BusinessServices.Scheduling;
using NQueue.BusinessServices.Scheduling.ScheduleProcessors;

namespace NQueue
{
    public partial class JobScheduleControl : NQueue.BaseEntityControl
    {
        public JobScheduleControl(bool newItem, RefreshRequired refreshRequired)
            : base(newItem, refreshRequired)
        {
            InitializeComponent();
        }

        protected override void Add()
        {
            ScheduleManager.InsertInstanceSchedule(
                ((JobInstance)cboJobInstances.SelectedItem).JobInstanceID,
                ((ScheduleType)cboScheduleTypes.SelectedItem).ScheduleTypeID,
                scheduleProperties.SelectedObject);
        }

        protected override void Delete()
        {
            ScheduleManager.DeleteInstanceSchedule(_schedule);
        }

        private JobInstanceSchedule _schedule;

        private void JobScheduleControl_Load(object sender, EventArgs e)
        {
            cboJobInstances.DataSource = JobManager.GetAllJobInstances();
            cboScheduleTypes.DataSource = ScheduleManager.GetAllScheduleTypes();

            if (!NewItem)
            {
                _schedule = (JobInstanceSchedule)ControllableItem;

                lblJobScheduleID.Text = _schedule.JobInstanceScheduleID.ToString();
                foreach (JobInstance instance in cboJobInstances.Items)
                {
                    if (instance.JobInstanceID == _schedule.JobInstanceID)
                    {
                        cboJobInstances.SelectedItem = instance;
                    }
                }
                foreach (ScheduleType scheduleType in cboScheduleTypes.Items)
                {
                    if (scheduleType.ScheduleTypeID == _schedule.ScheduleTypeID)
                    {
                        cboScheduleTypes.SelectedItem = scheduleType;
                        scheduleProperties.SelectedObject = ScheduleManager.GetScheduleProcessor(scheduleType).DeserialiseParametersObject(_schedule.ScheduleParameters);
                    }
                }
            }
            else
            {
                cboJobInstances.Enabled = true;
                cboScheduleTypes.Enabled = true;
            }
        }

        private void cboScheduleTypes_SelectedIndexChanged(object sender, EventArgs e)
        {
            ScheduleProcessor scheduleProcessor = ScheduleManager.GetScheduleProcessor(((ScheduleType)cboScheduleTypes.SelectedItem).ScheduleTypeID);
            scheduleProperties.SelectedObject = scheduleProcessor.GetBlankParametersObject();
        }


    }
}

