using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NQueue.Entities;
using NQueue.BusinessServices.System;
using NQueue.BusinessServices.Jobs;
using NQueue.Client;

namespace NQueue
{
    public partial class NQueueUIForm : Form
    {
        public NQueueUIForm()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            EventLogger.LogInformationEvent("NQueue Administration Console loaded");
            SplashScreen splash = new SplashScreen("NQueue");
            splash.Show();
            try
            {
                RefreshConfigurationTreeContents();
            }
            finally
            {
                splash.Loaded();
            }
            
        }

        private void RefreshConfigurationTreeContents()
        {
            try
            {
                //get the entire system configuration out of the database
                SystemConfiguration configuration = SystemManager.GetSystemConfiguration();
                TreeNode rootNode = treeConfigItems.Nodes[0];
                TreeNode hostNode = rootNode.Nodes[0];
                TreeNode hostInstanceNode = rootNode.Nodes[1];
                TreeNode jobDefinitionNode = rootNode.Nodes[2];
                TreeNode jobInstanceNode = rootNode.Nodes[3];

                hostNode.Nodes.Clear();

                hostInstanceToolStripMenuItem.Enabled = false;
                jobInstanceToolStripMenuItem.Enabled = false;
                scheduleToolStripMenuItem.Enabled = false;

                if (configuration.Hosts.Count > 0)
                {
                    hostInstanceToolStripMenuItem.Enabled = true;
                }
                if (configuration.JobDefinitions.Count > 0)
                {
                    jobInstanceToolStripMenuItem.Enabled = true;
                }
                if (configuration.JobInstances.Count > 0)
                {
                    scheduleToolStripMenuItem.Enabled = true;
                }
                


                foreach (Host host in configuration.Hosts)
                {
                    AddTreeNode(hostNode, host.HostName, host, new HostControl(false, RefreshConfigurationTreeContents));
                }

                hostInstanceNode.Nodes.Clear();
                foreach (HostInstance hostInstance in configuration.HostInstances)
                {
                    AddTreeNode(hostInstanceNode, string.Format("{0}:{1}", hostInstance.HostName, hostInstance.ServerName), hostInstance, new HostInstanceControl(false, RefreshConfigurationTreeContents));
                }

                jobDefinitionNode.Nodes.Clear();
                foreach (JobDefinition jobDef in configuration.JobDefinitions)
                {
                    AddTreeNode(jobDefinitionNode, jobDef.JobName, jobDef, new JobDefinitionControl(false, RefreshConfigurationTreeContents));
                }

                jobInstanceNode.Nodes.Clear();
                foreach (JobInstance jobInstance in configuration.JobInstances)
                {
                    TreeNode instanceNode = AddTreeNode(jobInstanceNode, jobInstance.JobInstanceName, jobInstance, new JobInstanceControl(false, RefreshConfigurationTreeContents));
                    //for each job instance, add all schedules
                    foreach (JobInstanceSchedule schedule in JobManager.GetSchedulesForInstance(jobInstance))
                    {
                        AddTreeNode(instanceNode, schedule.ScheduleTypeName, schedule, new JobScheduleControl(false, RefreshConfigurationTreeContents));
                    }
                }

                mainSplitContainer.Panel2.Controls.Clear();

            }
            catch (Exception exc)
            {

                MessageBox.Show("Exception: " + exc.Message, "Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private static TreeNode AddTreeNode(TreeNode rootNode, string name, object item, BaseEntityControl editingControl)
        {
            TreeNode newNode = new TreeNode(name);
            editingControl.SetItem(item);
            newNode.Tag = editingControl;
            rootNode.Nodes.Add(newNode);
            return newNode;
        }


        private void treeConfigItems_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node.Tag is BaseEntityControl)
            {
                mainSplitContainer.Panel2.Controls.Clear();
                mainSplitContainer.Panel2.Controls.Add((BaseEntityControl)e.Node.Tag);
                mainSplitContainer.Panel2.Controls[0].Dock = DockStyle.Fill;
            }
        }
        private void ShowCombinedServiceControl()
        {
            mainSplitContainer.Panel2.Controls.Clear();
            mainSplitContainer.Panel2.Controls.Add(new CombinedServiceControl());
            mainSplitContainer.Panel2.Controls[0].Dock = DockStyle.Fill;
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RefreshConfigurationTreeContents();
        }

        private void hostToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddNewItem newItemDialog = new AddNewItem(new HostControl(true, RefreshConfigurationTreeContents));
            newItemDialog.ShowDialog();
        }

        private void hostInstanceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddNewItem newItemDialog = new AddNewItem(new HostInstanceControl(true, RefreshConfigurationTreeContents));
            newItemDialog.ShowDialog();
        }

        private void jobDefintionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddNewItem newItemDialog = new AddNewItem(new JobDefinitionControl(true, RefreshConfigurationTreeContents));
            newItemDialog.ShowDialog();
        }

        private void jobInstanceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddNewItem newItemDialog = new AddNewItem(new JobInstanceControl(true, RefreshConfigurationTreeContents));
            newItemDialog.ShowDialog();
        }

        private void scheduleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddNewItem newItemDialog = new AddNewItem(new JobScheduleControl(true, RefreshConfigurationTreeContents));
            newItemDialog.ShowDialog();
        }

        private void showCombinedServiceControllerMenuItem_Click(object sender, EventArgs e)
        {
            ShowCombinedServiceControl();
        }







    }
}