using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace NQueue
{
    public partial class TypeBrowserForm : Form
    {

        /// <summary>
        /// Fired when a user double clicks a type node in the browser
        /// </summary>
        public event NQueue.TypeBrowser.TypeSelectedDelegate TypeSelected;
        private Type _onlyDisplayInheritorsOf;

        public TypeBrowserForm(Type onlyDisplayInheritorsOf)
        {
            this._onlyDisplayInheritorsOf = onlyDisplayInheritorsOf;
            Initialise();
        }

        public TypeBrowserForm()
        {
            Initialise();
        }

        private void Initialise()
        {
            InitializeComponent();
            typeBrowser.TypeSelected += new TypeBrowser.TypeSelectedDelegate(typeBrowser_TypeSelected);
        }

        void typeBrowser_TypeSelected(string assemblyFullName, string typeFullName)
        {
            if (TypeSelected != null)
                TypeSelected(assemblyFullName, typeFullName);
            this.Close();
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            openFile.Filter = "Assemblies (*.dll)|*.dll";
            openFile.Title = "Select assembly containing type";
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                txtPath.Text = openFile.FileName;
            }
        }

        private void btnSelect_Click(object sender, EventArgs e)
        {
            if (!File.Exists(txtPath.Text))
            {
                MessageBox.Show("Cannot find file at: " + txtPath.Text);
                return;
            }
            else
            {
                typeBrowser.DisplayAssembly(txtPath.Text, true, _onlyDisplayInheritorsOf);
            }


        }
    }
}