using System;
using System.Collections.Generic;
using System.Text;
using NQueue.BusinessServices.Jobs;

namespace NQueueCmd
{
    class Program
    {
        static void Main(string[] args)
        {
            ConsoleColor originalColour = Console.ForegroundColor;
            try
            {
                Console.ForegroundColor = ConsoleColor.Cyan;
                Console.WriteLine("NQueue CommandLine Utility");
                if (args.Length == 1 && args[0] == "?")
                {
                    Usage();
                    Environment.Exit(0);
                }
                if (args.Length < 1 || args.Length > 2)
                {
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.WriteLine("Incorrect number of parameters supplied; " + args.Length.ToString());
                    Usage();
                }
                else
                {
                    string jobInstanceName = args[0];
                    Console.WriteLine(string.Format("Enqueuing job instance {0} for immediate execution", jobInstanceName));
                    string serverAffinity = null;
                    if (args.Length > 1)
                    {
                        string serverAffinityArgument = args[1].ToLower().Trim();
                        if (serverAffinityArgument == "(local)")
                        {
                            serverAffinity = System.Environment.MachineName;
                        }
                        else
                        {
                            serverAffinity = serverAffinityArgument;
                        }
                        //check we can run on this host instance
                        if (!JobManager.CanJobInstanceExecuteOnServer(jobInstanceName, serverAffinity))
                            throw new InvalidOperationException("Cannot enqueue job on this server - the job does not have an active Host Instance running on this server");

                        Console.WriteLine(string.Format("Server affinity set to {0} - job will execute on {1}", serverAffinityArgument, serverAffinity));
                    }
                    JobManager.EnqueueJobInstanceNow(jobInstanceName, serverAffinity);
                    Console.WriteLine("Job instance enqueued");
                    Environment.ExitCode = 0;
                }
            }
            catch (Exception exc)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("Exception occurred");
                Console.WriteLine(exc.Message);
                Console.WriteLine(exc.StackTrace);
                Environment.ExitCode = 1;
            }
            finally
            {
                Console.ForegroundColor = originalColour;
            }

        }

        private static void Usage()
        {
            Console.WriteLine("Usage: NQueueCmd.exe <JobInstanceName> [<ServerAffinity>]");
            Console.WriteLine("Set ServerAffinity to a server name to ensure the job executes on the given server.  Specify (local) to execute on the local machine");
        }
    }
}
