use NQueue
go

create procedure NQueue_HostInsert
	@HostName varchar(70)
as
	insert Hosts (HostName)
	values (@HostName)
	return @@identity
GO
grant exec on NQueue_HostInsert to NQueue_administrators
go

create procedure NQueue_HostDelete
	@HostID int
as
	update Hosts set Deleted = 1 where
	HostID = @HostID
go
grant exec on NQueue_HostDelete to NQueue_administrators
go



create procedure NQueue_HostsSelectAll
as
	select HostID, HostName from Hosts
	where Deleted = 0
go
grant exec on NQueue_HostsSelectAll to NQueue_system
go



create procedure NQueue_HostsSelectOne
	@HostID int
as
	select HostID, HostName from Hosts where
	HostID = @HostID 
	and Deleted = 0
go
grant exec on NQueue_HostsSelectOne to NQueue_system
go



create procedure NQueue_HostInstanceInsert
	@HostID int,
	@ServerName varchar(256)
as	
	insert HostInstances (HostID, ServerName)
	values (@HostID, @ServerName)
	return @@identity
go
grant exec on NQueue_HostInstanceInsert to NQueue_administrators
go


create procedure NQueue_HostInstanceDelete
	@HostInstanceID int
as
	update HostInstances set Deleted = 1 where
	HostInstanceID = @HostInstanceID
go
grant exec on NQueue_HostInstanceDelete to NQueue_administrators
go

create procedure NQueue_HostInstancesSelectOne
	@HostInstanceID int
as
	select hi.HostID, hi.ServerName, h.HostName from HostInstances hi 
	inner join Hosts h on hi.HostID = h.HostID
	where HostInstanceID = @HostInstanceID
	and hi.Deleted = 0
go
grant exec on NQueue_HostInstancesSelectOne to NQueue_system
go

create procedure NQueue_HostInstancesSelectAll
as
	select hi.HostInstanceID, hi.HostID, hi.ServerName, h.HostName
	from HostInstances hi inner join Hosts h on
	hi.HostID = h.HostID
	where hi.Deleted = 0
go
grant exec on NQueue_HostInstancesSelectAll to NQueue_system
go

create procedure NQueue_HostInstancesSelectAllForHost
	@HostID int
as
	select hi.HostInstanceID, hi.HostID, hi.ServerName, h.HostName
	from HostInstances hi inner join Hosts h on
	hi.HostID = h.HostID
	where hi.HostID = @HostID
	and hi.Deleted = 0
go
grant exec on NQueue_HostInstancesSelectAllForHost to NQueue_system
go

create procedure NQueue_JobDefinitionInsert
	@JobName varchar(70),
	@JobDescription varchar(300),
	@JobAssemblyStrongName nvarchar(256),
	@JobClassType nvarchar(256),
	@JobConfigFile nvarchar(max)
as
	insert JobDefinitions
	(
	JobName,
	JobDescription,
	JobAssemblyStrongName,
	JobClassType,
	JobConfigFile
	)
	values
	(
	@JobName,
	@JobDescription,
	@JobAssemblyStrongName,
	@JobClassType,
	@JobConfigFile	
	)
	return @@identity
go
grant exec on NQueue_JobDefinitionInsert to NQueue_administrators
go

create procedure NQueue_JobDefinitionDelete
	@JobDefinitionID int
as
	update JobDefinitions set deleted = 1
	where JobDefinitionID = @JobDefinitionID
go
grant exec on NQueue_JobDefinitionDelete to NQueue_administrators
go

create procedure NQueue_JobDefinitionSelectAll
as
	select JobDefinitionID, JobName, JobDescription, JobAssemblyStrongName, JobClassType, JobConfigFile from JobDefinitions
	where deleted = 0
go
grant exec on NQueue_JobDefinitionSelectAll to NQueue_system
go

create procedure NQueue_JobDefinitionsSelectOne
	@JobDefinitionID int
as
	select JobDefinitionID, JobName, JobDescription, JobAssemblyStrongName, JobClassType, JobConfigFile from JobDefinitions
	where JobDefinitionID = @JobDefinitionID
	and deleted = 0
go
grant exec on NQueue_JobDefinitionsSelectOne to NQueue_system
go

create procedure NQueue_JobInstanceInsert
	@JobDefinitionID int,
	@JobInstanceName varchar(70),
	@HostID int,
	@InstanceTTLTimeSpan int,
	@JobParameters nvarchar(max),
	@Singleton bit
as

	declare @instanceWithNameCount int;
	select @instanceWithNameCount = count(*) from 
	JobInstances where JobInstanceName = @JobInstanceName
	and deleted = 0
	
	if @instanceWithNameCount > 0
	begin
		RAISERROR ('A job instance with that name already exists', 16, 1)
		return -1
	end
	
	insert JobInstances
	(
		JobDefinitionID,
		JobInstanceName,
		HostID,
		InstanceTTLTimeSpan,
		JobParameters,
		Singleton 
	)	
	values
	(
		@JobDefinitionID,
		@JobInstanceName,
		@HostID,
		@InstanceTTLTimeSpan,
		@JobParameters ,
		@Singleton
	)
	return @@identity
go
grant exec on NQueue_JobInstanceInsert to NQueue_administrators
go

create procedure NQueue_JobInstanceDelete
	@JobInstanceID int
as
	update JobInstances set deleted = 1 where
	JobInstanceID = @JobInstanceID
go
grant exec on NQueue_JobInstanceDelete to NQueue_administrators
go

create procedure NQueue_JobInstanceSelectOne
	@JobInstanceID int
as
	select ji.JobInstanceID, ji.JobDefinitionID, ji.JobInstanceName, ji.HostID, ji.InstanceTTLTimeSpan, ji.JobParameters, jd.JobName, ji.JobInstanceStateID, ji.Singleton
	from JobInstances ji inner join JobDefinitions jd on ji.JobDefinitionID = jd.JobDefinitionID
	where JobInstanceID = @JobInstanceID
	and ji.deleted = 0
go
grant exec on NQueue_JobInstanceSelectOne to NQueue_system
go

create procedure NQueue_JobInstanceSelectAll
as
	select ji.JobInstanceID, ji.JobDefinitionID, ji.JobInstanceName, ji.HostID, ji.InstanceTTLTimeSpan, ji.JobParameters, jd.JobName, ji.JobInstanceStateID, ji.Singleton 
	from JobInstances ji inner join JobDefinitions jd on ji.JobDefinitionID = jd.JobDefinitionID
	where ji.deleted = 0
go
grant exec on NQueue_JobInstanceSelectAll to NQueue_system
go

create procedure NQueue_JobInstanceSelectAllForJob
	@JobDefinitionID int
as
	select ji.JobInstanceID, ji.JobDefinitionID, ji.JobInstanceName, ji.HostID, ji.InstanceTTLTimeSpan, ji.JobParameters, jd.JobName, ji.JobInstanceStateID, ji.Singleton 
	from JobInstances ji inner join JobDefinitions jd on ji.JobDefinitionID = jd.JobDefinitionID
	where ji.JobDefinitionID = @JobDefinitionID
	and ji.deleted = 0
go
grant exec on NQueue_JobInstanceSelectAll to NQueue_system
go

create procedure NQueue_JobInstanceUpdateState
	@JobInstanceID int,
	@JobInstanceStateID int
as
	update JobInstances set 
	JobInstanceStateID = @JobInstanceStateID
	where JobInstanceID = @JobInstanceID
go
grant exec on NQueue_JobInstanceUpdateState to NQueue_system
go


	


---work queue stuff

create procedure NQueue_WorkQueue_EnqueueWorkItem
	@JobInstanceID int,
	@DateToExecute datetime,
	@ServerAffinity varchar(256) = null
as
	insert WorkQueue (JobInstanceID, ServerAffinity, DateEnqueued, DateToExecute, WorkItemStatusID)
	values (@JobInstanceID, @ServerAffinity, GetDate(), @DateToExecute, 1)	--1 = new
	return @@identity
go
grant exec on NQueue_WorkQueue_EnqueueWorkItem to NQueue_system
go

create procedure NQueue_WorkQueue_DequeueWorkItem
	@HostInstanceID int,
	@ServerName varchar(256) = null
as
	set nocount on
	set transaction isolation level read committed 
	set deadlock_priority low
	
	declare @TargetWorkItemID bigint
	declare @TargetJobInstanceID int
	declare @HostID int
	
	select @HostID = HostID from HostInstances where HostInstanceID = @HostInstanceID
	
	begin transaction
	-- get an item off the top of the queue
	select top 1
	@TargetWorkItemID = wq.WorkItemID, @TargetJobInstanceID = wq.JobInstanceID
	from WorkQueue wq with (updlock rowlock readpast)
	inner join JobInstances ji
	on wq.JobInstanceID = ji.JobInstanceID
	where ji.HostID = @HostID 
	and ji.JobInstanceStateID <> 3 -- 3 = paused (paused items should not be dequeued)
	and wq.WorkItemStatusID = 1 --1 = new
	and wq.DateToExecute < GetDate()
	and (@ServerName is null or wq.ServerAffinity is null or @ServerName = wq.ServerAffinity)
	order by wq.WorkItemID asc --oldest first
	
	if @TargetWorkItemID is not null
	begin
		update WorkQueue
		set HostInstanceID = @HostInstanceID,
		WorkItemStatusID = 2,	--2 = processing
		DateToExpire = dateadd(mi, ji.InstanceTTLTimeSpan, GetDate()),
		DateDequeued = GetDate()
		from JobInstances ji
		where WorkItemID = @TargetWorkItemID
		and ji.JobInstanceID = @TargetJobInstanceID
	end	
	if @@ERROR <> 0
	begin
		rollback
		RAISERROR ('Error dequeuing work item.', 16, 1)
		return
	end
	commit transaction
	--return the dequeued work item
	if @TargetWorkItemID is not null
	begin
		select @TargetWorkItemID as WorkItemID, ji.JobInstanceID, ji.JobDefinitionID, ji.JobInstanceName, ji.JobParameters, 
		jd.JobName, jd.JobAssemblyStrongName, jd.JobClassType, jd.JobConfigFile, ji.JobInstanceStateID, ji.Singleton, ji.InstanceTTLTimeSpan
		from JobInstances ji
		inner join JobDefinitions jd 
		on ji.JobDefinitionID = jd.JobDefinitionID
		where ji.JobInstanceID = @TargetJobInstanceID
	end
		
go
	
grant exec on NQueue_WorkQueue_DequeueWorkItem to NQueue_system
go

create procedure NQueue_WorkQueue_MarkItemCompleted
	@WorkItemID bigint,
	@WorkItemStatusID int
as
	update WorkQueue
	set WorkItemStatusID = @WorkItemStatusID,
	DateWorkCompleted = GetDate()
	where WorkItemID = @WorkItemID
go
grant exec on NQueue_WorkQueue_MarkItemCompleted to NQueue_system
go


create procedure NQueue_JobInstance_InsertSchedule
	@JobInstanceID int,
	@ScheduleTypeID int,
	@ScheduleParameters varchar(max)
as
	insert JobInstanceSchedule
	(JobInstanceID, ScheduleTypeID, ScheduleParameters)
	values
	(@JobInstanceID, @ScheduleTypeID, @ScheduleParameters)
	return @@identity
go
grant exec on NQueue_JobInstance_InsertSchedule to NQueue_system
go


create procedure NQueue_JobInstance_DeleteSchedule
	@JobInstanceScheduleID int
as
	update JobInstanceSchedule set deleted = 1
	where JobInstanceScheduleID = @JobInstanceScheduleID
go
grant exec on NQueue_JobInstance_DeleteSchedule to NQueue_system
go


create procedure NQueue_JobInstanceSchedule_GetForJob
	@JobInstanceID int
as
	select jis.JobInstanceScheduleID, jis.JobInstanceID, jis.ScheduleTypeID, jis.InstanceScheduleLastRun, jis.ScheduleParameters, st.ScheduleType
	from JobInstanceSchedule jis 
	inner join ScheduleTypes st on 
	jis.ScheduleTypeID = st.ScheduleTypeID
	where
	jis.JobInstanceID = @JobInstanceID
	and jis.deleted = 0
go
grant exec on NQueue_JobInstanceSchedule_GetForJob to NQueue_system
go

create procedure NQueue_JobInstanceSchedule_GetForScheduleType
	@ScheduleTypeID int
as
	select jis.JobInstanceScheduleID, jis.JobInstanceID, jis.ScheduleTypeID, jis.InstanceScheduleLastRun, jis.ScheduleParameters, st.ScheduleType
	from JobInstanceSchedule jis 
	inner join ScheduleTypes st on 
	jis.ScheduleTypeID = st.ScheduleTypeID
	where 
	jis.ScheduleTypeID = @ScheduleTypeID
	and jis.deleted  = 0
go
grant exec on NQueue_JobInstanceSchedule_GetForScheduleType to NQueue_system
go

create procedure NQueue_JobInstanceSchedule_MarkLastProcessed
	@JobInstanceScheduleID int
as
	update JobInstanceSchedule set InstanceScheduleLastRun = GetDate()
	where JobInstanceScheduleID = @JobInstanceScheduleID
go
grant exec on NQueue_JobInstanceSchedule_MarkLastProcessed to NQueue_system
go

create procedure NQueue_JobSchedule_GetAllSchedules
as
	select ScheduleTypeID, ScheduleType, CurrentlyProcessing, LastProcessed, NextProcessDue, ProcessingHostInstanceID, ScheduleTypeStrongName, ScheduleTypeClassType
	from ScheduleTypes
	
go
grant exec on NQueue_JobSchedule_GetAllSchedules to NQueue_system
go

create procedure NQueue_JobSchedule_GetScheduleType
	@ScheduleTypeID int
as
	select ScheduleTypeID, ScheduleType, CurrentlyProcessing, LastProcessed, NextProcessDue, ProcessingHostInstanceID, ScheduleTypeStrongName, ScheduleTypeClassType
	from ScheduleTypes
	where ScheduleTypeID = @ScheduleTypeID
	
go
grant exec on NQueue_JobSchedule_GetScheduleType to NQueue_system
go

create procedure NQueue_JobSchedule_MarkScheduleTypeAsBeingProcessed
	@ScheduleTypeID int,
	@LastProcessedDate datetime,
	@ProcessingHostInstanceID int
as

	set nocount on
	set transaction isolation level read committed 
	set deadlock_priority low

	begin transaction

    declare @LastProcessedRecorded datetime, @ProcessingInstanceIDRecorded int, @CurrentlyProcessingRecorded bit
	select @CurrentlyProcessingRecorded = CurrentlyProcessing, @LastProcessedRecorded = LastProcessed, @ProcessingInstanceIDRecorded = ProcessingHostInstanceID
	from ScheduleTypes with (updlock rowlock)
	where ScheduleTypeID = @ScheduleTypeID
	
	if @LastProcessedRecorded <> @LastProcessedDate or @ProcessingInstanceIDRecorded <> null or @CurrentlyProcessingRecorded = 1
	begin
		rollback
		RAISERROR ('Concurrency violation', 16, 1)
		return
	end
	
	update ScheduleTypes set
	CurrentlyProcessing = 1,
	ProcessingHostInstanceID = @ProcessingHostInstanceID
	where ScheduleTypeID = @ScheduleTypeID
	
	if @@ERROR <> 0
	begin
		rollback
		RAISERROR ('Error marking schedule as being processed.', 16, 1)
		return
	end
	commit transaction
go
grant exec on NQueue_JobSchedule_MarkScheduleTypeAsBeingProcessed to NQueue_system
go	

create procedure NQueue_JobSchedule_RemoveProcessMark
	@ScheduleTypeID int,
	@ProcessingHostInstanceID int
as
	set nocount on
	set transaction isolation level read committed 

	update ScheduleTypes set 
	CurrentlyProcessing = 0, 
	ProcessingHostInstanceID = null
	where ScheduleTypeID = @ScheduleTypeID
	and ProcessingHostInstanceID = @ProcessingHostInstanceID
go

grant exec on NQueue_JobSchedule_RemoveProcessMark to NQueue_system
go	


create procedure NQueue_JobSchedule_ShowScheduleProcessed
	@ScheduleTypeID int,
	@LastProcessedDate datetime,
	@NextProcessingDate datetime,
	@ProcessingHostInstanceID int
as
	set nocount on
	set transaction isolation level read committed 
	set deadlock_priority low

    declare @TranCount int
    set @TranCount = @@trancount

	if @TranCount = 0 
		begin transaction 

    declare @LastProcessedRecorded datetime, @ProcessingInstanceIDRecorded int, @CurrentlyProcessingRecorded bit
	select @CurrentlyProcessingRecorded = CurrentlyProcessing, @LastProcessedRecorded = LastProcessed, @ProcessingInstanceIDRecorded = ProcessingHostInstanceID
	from ScheduleTypes with (updlock rowlock)
	where ScheduleTypeID = @ScheduleTypeID

	if @LastProcessedRecorded <> @LastProcessedDate or @ProcessingInstanceIDRecorded <> @ProcessingHostInstanceID or @CurrentlyProcessingRecorded = 0
	begin
		RAISERROR ('Concurrency violation', 16, 1)
		return
	end

	update ScheduleTypes set 
	CurrentlyProcessing = 0, 
	LastProcessed = GetDate(), 
	NextProcessDue = @NextProcessingDate, 
	ProcessingHostInstanceID = null
	where ScheduleTypeID = @ScheduleTypeID

	if @@ERROR <> 0
	begin
		RAISERROR ('Error marking schedule as finished processing.', 16, 1)
		return
	end

	if @TranCount = 0 
		commit transaction
go
grant exec on NQueue_JobSchedule_ShowScheduleProcessed to NQueue_system
go	


create procedure NQueue_WorkLog_AddLogItem
	@WorkItemID bigint,
	@WorkLogTypeID int,
	@StatusDescription nvarchar (1000),
	@ExceptionStackTrace nvarchar(max) = null
as
	insert WorkLog
	(
		WorkItemID,
		WorkLogTypeID,
		StatusDescription,
		ExceptionStackTrace
	)
	values
	(
		@WorkItemID,
		@WorkLogTypeID,
		@StatusDescription,
		@ExceptionStackTrace
	)
go

grant exec on NQueue_WorkLog_AddLogItem to NQueue_system
go

create procedure NQueue_GetJobInstanceByName
	@JobInstanceName varchar(70)
as
	select ji.JobInstanceID, ji.JobDefinitionID, ji.JobInstanceName, ji.HostID, ji.InstanceTTLTimeSpan, ji.JobParameters, jd.JobName, ji.JobInstanceStateID, ji.Singleton
	from JobInstances ji inner join JobDefinitions jd on ji.JobDefinitionID = jd.JobDefinitionID
	where ji.JobInstanceName = @JobInstanceName
	and ji.deleted = 0
go

grant exec on NQueue_GetJobInstanceByName to NQueue_system
go

create procedure NQueue_ExpireMutexes
as
	update Mutex
	set ServerWithLock = null
	where ExpiryDate < GetDate()
	and ServerWithLock is not null
go

grant exec on NQueue_ExpireMutexes to NQueue_system
go

create procedure NQueue_ObtainMutex
	@MutexName varchar(500),
	@ServerName varchar(256),
	@ExpiryDate datetime
as
	declare @MutexID int
	declare @ServerWithLock varchar(256)
	
	--expire any old mutexes
	exec NQueue_ExpireMutexes
	--does a row already exist for the mutex?  Try and select it and take
	--an update lock on the table so that another process cannot change 
	--@ServerWithLock before we attempt to update it
	select @MutexID = MutexID, @ServerWithLock = ServerWithLock from Mutex 
	with (updlock rowlock)
	where @MutexName = MutexName
	
	if @MutexID is null	--mutex was not found, so create it
	begin
		insert Mutex (MutexName, ServerWithLock, ExpiryDate)
		values (@MutexName, @ServerName, @ExpiryDate)
		return @@identity
	end
	else
	begin
		if @ServerWithLock is null	--mutex was found and was free to use
		begin
			update Mutex set ServerWithLock = @ServerName, ExpiryDate = @ExpiryDate where 
			MutexID = @MutexID
			return @MutexID
		end
		else
		begin
			--another server had the mutex
			return 0
		end	
	end
go

grant exec on NQueue_ObtainMutex to NQueue_system
go

--releases a mutex for use by another process
create procedure NQueue_ReleaseMutex
	@MutexID int,
	@ServerName varchar(256)
as
	declare @ServerWithLock varchar(256)
	select @ServerWithLock = ServerWithLock from Mutex 
	with (updlock rowlock)
	where @MutexID = MutexID
	
	if @ServerWithLock <> @ServerName and @ServerWithLock is not null
	begin
		RAISERROR ('Attempted to release a lock that was not held by the server.', 16, 1)
	end

	update Mutex
	set ServerWithLock = null where 
	MutexID = @MutexID
go
grant exec on NQueue_ReleaseMutex to NQueue_system
go


--deletes a mutex entirely
create procedure NQueue_DeleteMutex
	@MutexID int,
	@ServerName varchar(256)
as

	declare @ServerWithLock varchar(256)
	select @ServerWithLock = ServerWithLock from Mutex 
	with (updlock rowlock)
	where @MutexID = MutexID
	
	if @ServerWithLock <> @ServerName and @ServerWithLock is not null
	begin
		RAISERROR ('Attempted to delete a lock that was not held by the server.', 16, 1)
	end

	delete Mutex where
	MutexID = @MutexID
go

create procedure NQueue_GetAllWorkItemsForJobInstance
	@JobInstanceID int
as
	select WorkItemID, HostInstanceID, JobInstanceID, ServerAffinity, DateEnqueued, DateToExecute, 
	DateDequeued, DateToExpire, DateWorkCompleted, WorkItemStatusID
	from WorkQueue where
	JobInstanceID = @JobInstanceID
	order by DateEnqueued desc
go
grant exec on NQueue_GetAllWorkItemsForJobInstance to NQueue_system
go


create procedure NQueue_GetWorkLogForWorkItem
	@WorkItemID bigint
as
	select WorkLogItemID, WorkItemID, WorkLogTypeID, LogTime, StatusDescription, ExceptionStackTrace
	from WorkLog where WorkItemID = @WorkItemID
	order by LogTime asc
go

grant exec on NQueue_GetWorkLogForWorkItem to NQueue_system
go








--- Site map stored procs and triggers
create PROCEDURE proc_GetSiteMap 
AS 


declare @tempSiteMap table(
    ID          int identity (1, 1) primary key not null,
    NativeID	int not null,
    Title       varchar (70),
    Description varchar (512),
    Url         varchar (512),
    Roles       varchar (512),
    Parent      int
) 


insert into @tempSiteMap (NativeID, Title, Description, Url, Roles, Parent)
values (0, 'Jobs', 'All defined jobs', null, null, null)

insert into @tempSiteMap (NativeID, Title, Description, Url, Roles, Parent)
select JobDefinitionID, JobName, JobDescription, null, null, 1 
from JobDefinitions where Deleted = 0

insert into @tempSiteMap (NativeID, Title, Description, Url, Roles, Parent)
select ji.JobInstanceID, ji.JobInstanceName + ' (' + jis.JobInstanceStateName + ')', ji.JobInstanceName, '~/JobInstanceReport.aspx?JobInstanceID=' + cast (ji.JobInstanceID as varchar(16)), null, sm.ID
from JobInstances ji
inner join JobInstanceStates jis on ji.JobInstanceStateID = jis.JobInstanceStateID
inner join @tempSiteMap sm on ji.JobDefinitionID = sm.NativeID
where ji.Deleted = 0


select ID, Title, Description, Url, Roles, Parent from @tempSiteMap

go

grant exec on proc_GetSiteMap to NQueue_system
go



/****** Object:  StoredProcedure [dbo].[AspNet_SqlCachePollingStoredProcedure]    Script Date: 06/13/2008 14:24:25 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[AspNet_SqlCachePollingStoredProcedure] AS
         SELECT tableName, changeId FROM dbo.AspNet_SqlCacheTablesForChangeNotification
         RETURN 0
GO
/****** Object:  StoredProcedure [dbo].[AspNet_SqlCacheUpdateChangeIdStoredProcedure]    Script Date: 06/13/2008 14:24:25 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [dbo].[AspNet_SqlCacheUpdateChangeIdStoredProcedure] 
             @tableName NVARCHAR(450) 
         AS

         BEGIN 
             UPDATE dbo.AspNet_SqlCacheTablesForChangeNotification WITH (ROWLOCK) SET changeId = changeId + 1 
             WHERE tableName = @tableName
         END
GO
/****** Object:  Trigger [JobInstances_AspNet_SqlCacheNotification_Trigger]    Script Date: 06/13/2008 14:24:34 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE TRIGGER [dbo].[JobInstances_AspNet_SqlCacheNotification_Trigger] ON [dbo].[JobInstances]
                       FOR INSERT, UPDATE, DELETE AS BEGIN
                       SET NOCOUNT ON
                       EXEC dbo.AspNet_SqlCacheUpdateChangeIdStoredProcedure N'JobInstances'
                       END
GO


grant exec on [AspNet_SqlCachePollingStoredProcedure] to NQueue_system
go
grant exec on [AspNet_SqlCacheUpdateChangeIdStoredProcedure] to NQueue_system
go